function [sgncmb] = channelcombination(sgncmb, datachannel)

% CHANNELCOMBINATION creates a cell-array with combinations of EEG/MEG
% channels for subsequent cross-spectral-density and coherence analysis
%
% You should specify channel combinations as a two-column cell array, 
% e.g. 
%   cfg.sgncmb = {  'EMG' 'MLF31'
%                   'EMG' 'MLF32'
%                   'EMG' 'MLF33' };
% to compare EMG with these three sensors, or 
%   cfg.sgncmb = { 'MEG' 'MEG' };
% to make all MEG combinations, or
%   cfg.sgncmb = { 'EMG' 'MEG' };
% to make all combinations between the EMG and all MEG channels.
%
% For each column, you can specify a mixture of real channel labels
% and of special strings that will be replaced by the corresponding
% channel labels. Channels that are not present in the raw datafile are
% automatically removed from the channel list.
% 
% See also CHANNELSELECTION

% Copyright (C) 2003, Robert Oostenveld
%
% $Log: channelcombination.m,v $
% Revision 1.8  2004/10/19 10:44:43  roboos
% fixed bug in planar-gradient channel detection if labels are shorter than 3 chars
%
% Revision 1.7  2004/08/26 07:16:11  jansch
% re-added to CVS-repository.
%
% Revision 1.5  2004/01/16 08:28:50  roberto
% fixed small bug for 'all'
%
% Revision 1.4  2004/01/06 12:43:47  roberto
% redesign by JM
%
% Revision 1.3  2003/10/29 09:02:31  roberto
% fixed bug in varible names, sgncmb vs channel
%
% Revision 1.2  2003/10/27 16:24:06  roberto
% added check for double channel combinations
%
% Revision 1.1  2003/10/27 16:02:13  roberto
% new implementation after an idea by JM
%

if ischar(sgncmb) & strcmp(sgncmb, 'all')
  % make all possible combinations of all channels
  sgncmb = {'all' 'all'};
end

% translate both columns and subsequently make all combinations
sgncmb1 = channelselection(sgncmb(:,1), datachannel);
sgncmb2 = channelselection(sgncmb(:,2), datachannel);

% compute indices of sgncmb1 and sgncmb2 relative to datachannel
[i,indx,indx1]=intersect(sgncmb1,datachannel);
[i,indx,indx2]=intersect(sgncmb2,datachannel);

% remove double occurrences of channels in either set of signals
sgncmb1 = unique(sgncmb1);
sgncmb2 = unique(sgncmb2);
indx1   = unique(indx1);
indx2   = unique(indx2);

% create a matrix in which all possible combinations are set to one
cmb = zeros(length(datachannel));
for ch1=1:length(indx1)
    for ch2=1:length(indx2)
        cmb(indx1(ch1),indx2(ch2))=1;
    end    
end

% check if there are planar gradiometers in the data
dum = char(datachannel);
if size(dum,2)>3
  if ~isempty(find(strcmp('_dV',(cellstr(dum(:,end-2:end)))))) & ...
     ~isempty(find(strcmp('_dH',(cellstr(dum(:,end-2:end)))))),
    disp('Planar gradiometers detected...')
    dum = dum([find(strcmp('_dV',(cellstr(dum(:,end-2:end)))));
      find(strcmp('_dH',(cellstr(dum(:,end-2:end)))))],:);
    for ch=1:size(dum,1)
      sel = find(strcmp(cellstr(dum(ch,1:5)),cellstr(dum(:,1:5))));
      if length(sel) == 1, error('Planar combinations are incomplete...');end;
    end
  end
end

% remove 'auto'-combinations and double occurrences
cmb           = tril(cmb,-1)+triu(cmb,1)';
[indx1,indx2] = find(cmb); 

% convert to channel-names
sgncmb = {};
for ch=1:length(indx1)
    sgncmb{end+1,1}=datachannel{indx1(ch)};
    sgncmb{end  ,2}=datachannel{indx2(ch)};    
end

        
