function [cfg] = definetrial(cfg);

% DEFINETRIAL defines the trials, i.e. the pieces of data that will be read
% in for preprocessing. Trials are defined by their begin and end sample 
% in the datafile and each trial has an offset that defines where the 
% relative t=0 point is for that trial.
%
% Use as
%   [cfg] = definetrial(cfg)
% where the configuration structure should contain either
%   cfg.trialdef   = structure with details of trial definition, see below
%   cfg.trialfun   = function name, see below
%
% A call to DEFINETRIAL results in the trial definition "trl" being added
% to the output configuration structure. The trials are defined according
% to the triggers, trials or other events inn the data, or from a
% user-specified Matlab function which returns "trl".
% 
% The trialdefinition "trl" is an Nx3 matrix. The first column contains
% the sample-indices of the begin of the trial relative to the begin
% of the raw data , the second column contains the sample_indices of
% the end of the trials, and the third column contains the offset of
% the trigger with respect to the trial. An offset of 0 means that
% the first sample of the trial corresponds to the trigger. A positive
% offset indicates that the first sample is later than the triger, a
% negative offset indicates a trial beginning before the trigger.
% 
% Simple trial definitions (e.g. based on a trigger alone) are supported by
% DEFINETRIAL itself. For this, the general and dataformat-independent way
% of handling trials is by relying on the READ_FCDC_EVENT function to
% collect all event information (such as triggers) from your dataset and
% select trials based on those events. This is implemented in DEFINETRIAL as
%   cfg.trialdef.eventtype  = 'string'
%   cfg.trialdef.eventvalue = number, string or list with numbers or strings
%   cfg.trialdef.prestim    = 0.300 latency in seconds (optional)
%   cfg.trialdef.poststim   = 0.700 latency in seconds (optional)
%
% If you specify 
%   cfg.trialdef.eventtype  = '?'
% a list with the events in your datafile will be displayed on screen.
%
% However, there are also many other complex ways in which you can define
% data pieces of interest, for example based on a conditional sequence of
% events (e.g. stimulus trigger followed by a correct response). For those
% cases, a general mechanism has been implemented through which you can
% supply your own trial-defining function, the 'trialfun'.
%
% This 'trialfun' is a string containing the name of a function that you
% have to write yourself. The function should take the cfg-structure as
% input and should give a Nx3 matrix in the same format as trl as the
% output. You can add extra custom fields to the cfg structure to pass as
% arguments to your own trialfun. Furthermore, inside the trialfun you can
% use the READ_FCDC_EVENT function to get the event information from your
% datafile.
%
% See also PREPROCESSING, READ_FCDC_HEADER, READ_FCDC_DATA, READ_FCDC_EVENT

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following method of selecting trials is still supported, but
% deprecated.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The old documentation follows:
%
% For some dataformats there are more specific ways of selecting trials. To
% use them, you should modify cfg.trialdef according to the following
% specifications (note that the numbers are only examples):
%
% For epoched CTF data, the trialdef structure should contain
%   cfg.trialdef.includeTrigger    = array, e.g. [3 7] 
%   cfg.trialdef.excludeTrigger    = array
%   cfg.trialdef.includeConditions = cell-array, e.g. {'C1', 'C2'}
%   cfg.trialdef.excludeConditions = cell-array, e.g. {'BAD'}
%   cfg.trialdef.prestim           = 0.300         latency in seconds
%   cfg.trialdef.poststim          = 0.700         latency in seconds
% Note that definitions according to the trigger values in the stimulus 
% channel can be combined with contitions defined in the CTF DataEditor.
% 
% For continuous CTF data, the trialdef structure should contain
%   cfg.trialdef.trigger  = 2             trigger code, can be an array
%   cfg.trialdef.prestim  = 0.300         latency in seconds
%   cfg.trialdef.poststim = 0.700         latency in seconds
%   cfg.trialdef.panel    = 'front' or 'back' (optional, back is the default)
% 
% For Neuromag data, the trialdef structure should contain
%   cfg.trialdef.trgchan  = channel label, e.g. 'STI 001'
%   cfg.trialdef.prestim  = 0.300         latency in seconds
%   cfg.trialdef.poststim = 0.700         latency in seconds
% 
% For continuous EEProbe data, the trialdef structure should contain
%   cfg.trialdef.trigger  = 2             trigger code, can be a list
%   cfg.trialdef.prestim  = 0.300         latency in seconds
%   cfg.trialdef.poststim = 0.700         latency in seconds
%   cfg.trialdef.trgfile  = filename of the trigger file
% 
% For continuous BrainVision data, the trialdef structure should contain
%   cfg.trialdef.stim     = 2             stimulus trigger code, can be a list
%   cfg.trialdef.resp     = 2             response trigger code, can be a list
%   cfg.trialdef.segment  = 'no' / 'yes'  use the segment markers as onset
%   cfg.trialdef.timezero = 'no' / 'yes'  use the "Time 0" markers as onset
%   cfg.trialdef.prestim  = 0.300         pre-marker latency in seconds
%   cfg.trialdef.poststim = 0.700         post-marker latency in seconds
%   cfg.trialdef.trgfile  = filename of the marker file
%
% For continuous NeuroScan data, the trialdef structure should contain
%   cfg.trialdef.trigger  = number or list with triggers
%   cfg.trialdef.prestim  = pre-stimulus in seconds
%   cfg.trialdef.poststim = post-stimulus in seconds
%
% For epoched NeuroScan data, the trialdef structure should not contain anything
%   cfg.trialdef = []
%
% For averaged EEProbe data, the trialdef structure should not contain anything
%   cfg.trialdef = []

% Copyright (c) 2003, Robert Oostenveld, F.C. Donders Centre
%
% $Log: definetrial.m,v $
% Revision 1.39  2005/02/16 15:18:25  roboos
% cleaned up the documentation, removed cfg.trialfile as option (was not supported)
% cleaned up the handling of events, retain event if already present in cfg (just like trl)
% removed unclear warning about old v.s. new style of configuration
% improved fprintf feedback on number of events and trials
%
% Revision 1.38  2005/02/02 12:03:21  roboos
% fixed bug for negative or zero trialduration
%
% Revision 1.37  2005/01/25 13:59:57  roboos
% added check for cfg.datatype=continous and extended the call to read_fcdc_data with the boundary check for non-continous data
%
% Revision 1.36  2004/11/15 09:18:44  roboos
% moved dataset to filename conversion into separate function (private/dataset2files)
% slight modification for handling of cfg.trialdef.eventvalue so that it also can handle strings as event values
% this is used for Neuroscan *.eeg files (value='accept' or 'reject')
%
% Revision 1.35  2004/10/01 09:57:10  roboos
% updated help: removed old fileformat specific way of handling events from the help
% the code still supports it but does print out a warning
% added option for printing out the events that were detected: cfg.trialdef.eventtype = '?'
% fixed bug in trl for eventtype and poststim: trial was one sample too long
%
% Revision 1.34  2004/09/27 16:04:57  roboos
% removed support for the obsoleted read_fcdc_trl format
%
% Revision 1.33  2004/09/27 15:37:11  roboos
% renamed the function readClassFile to read_ctf_cls (part of file_io)
%
% Revision 1.32  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%
% Revision 1.31  2004/08/25 13:12:17  roboos
% fixed begin/endsample for trldef.eventtype = 'trial' to synchronize it with recent change in read_fcdc_event
%
% Revision 1.30  2004/08/20 09:17:19  roboos
% changed order around: do nothing if trl is already present
%
% Revision 1.29  2004/08/19 10:39:03  roboos
% implemented general trialdefinition based on events from read_fcdc_event
% updated and restructured help
%
% Revision 1.28  2004/06/30 16:21:47  roboos
% improved handling of reading the event information from either the dataset, headerfile or datafile
%
% Revision 1.27  2004/06/24 10:12:42  roberto
% added warning if read_fcdc_event fails
%
% Revision 1.26  2004/06/03 15:49:22  roberto
% changed event handling, not added to cfg by default
%
% Revision 1.25  2004/06/03 08:31:38  roberto
% changed handling of events (added to cfg instead of extra output arg)
% fixed bug for eep_avr
%
% Revision 1.24  2004/05/17 16:01:21  roberto
% implemented event output as second output argument for trialfun
%

% if neccessary convert dataset into headerfile and datafile
cfg = dataset2files(cfg);

% read the header
hdr = read_fcdc_header(cfg.headerfile);

if ~isfield(cfg, 'datatype') | ~strcmp(cfg.datatype, 'continuous')
  % datatype is unknown or not continuous, perform epoch boundary check
  iscontinuous = 0;
else
  % do not perform epoch boundary check, usefull for pseudo-continuous data
  iscontinuous = strcmp(cfg.datatype, 'continuous');
end

if isfield(cfg, 'event')
  fprintf('retaining exist event information\n');
  event = cfg.event;
else
  % reading event information still is slightly experimental and not
  % supported for all file formats, therefore it is inside a try-end
  % statement
  try
    % read all the event information from the dataset
    if ~isempty(cfg.dataset)
      event = read_fcdc_event(cfg.dataset);
    elseif ~isempty(cfg.headerfile)
      event = read_fcdc_event(cfg.headerfile);
    elseif ~isempty(cfg.datafile)
      event = read_fcdc_event(cfg.datafile);
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% trial definition is already part of the configuration 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isfield(cfg, 'trl')
  fprintf('retaining exist trial definition\n');
  trl = cfg.trl;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% call the external trial function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialfun')
  % store the sampling frequency in the configuration
  % where it can be used for the external "trialfun" function
  cfg.fsample = hdr.Fs;
  
  if (nargout(cfg.trialfun)==1)
    % evaluate the user-defined function that gives back the trial definition
    trl = feval(cfg.trialfun, cfg);
  else     
    % include the optional "event" argument to receive a structure containing detailed 
    % information about conditions, reaction time or any other information
    [trl, event] = feval(cfg.trialfun, cfg);
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the general events structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialdef') & isfield(cfg.trialdef, 'eventtype')
  sel = [];
  trl = [];
  if strcmp(cfg.trialdef.eventtype, '?')
    % no trials should be added, show event information using subfunction and exit
    show_event(event);
    return
  end
  if ~isfield(cfg.trialdef, 'eventvalue')
    cfg.trialdef.eventvalue = [];
  elseif ischar(cfg.trialdef.eventvalue)
    % convert single string into cell-array, otherwise the intersection does not work as intended
    cfg.trialdef.eventvalue = {cfg.trialdef.eventvalue};
  end
  % select all events of the specified type and with the specified value
  for i=find(strcmp(cfg.trialdef.eventtype, {event.type}))
    if  isempty(cfg.trialdef.eventvalue)
      sel = [sel i];
    elseif ~isempty(intersect(event(i).value, cfg.trialdef.eventvalue))
      sel = [sel i];
    end
  end
  for i=sel
    % catch empty fields in the event table and interpret them meaningfully
    if isempty(event(i).offset)
      % time axis has no offset relative to the event
      event(i).offset = 0;      
    end
    if isempty(event(i).duration)
      % the event does not specify a duration
      event(i).duration = 0;
    end
    % determine where the trial starts with respect to the event
    if ~isfield(cfg.trialdef, 'prestim') 
      trloff = event(i).offset;
      trlbeg = event(i).sample;
    else
      % override the offset of the event
      trloff = round(-cfg.trialdef.prestim*hdr.Fs);
      % also shift the begin sample with the specified amount
      trlbeg = event(i).sample + trloff;
    end
    % determine the number of samples that has to be read (excluding the begin sample)
    if ~isfield(cfg.trialdef, 'poststim')
      trldur = max(event(i).duration - 1, 0);
    else
      trldur = round((cfg.trialdef.poststim+cfg.trialdef.prestim)*hdr.Fs) - 1;
    end
    trlend = trlbeg + trldur;
    % add the beginsample, endsample and offset of this trial to the list
    trl = [trl; [trlbeg trlend trloff]];
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the events in the EEProbe file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialdef') & isfield(cfg.trialdef, 'trgfile') & filetype(cfg.trialdef.trgfile, 'eep_trg')
  % read external trigger file in EEP format
  trg = read_eep_trg(cfg.trialdef.trgfile);
  trl = [];
  for i=1:length(trg)
    if ismember(trg(i).type,cfg.trialdef.trigger)
      trlbegin  = round((trg(i).time/1000-cfg.trialdef.prestim)*hdr.Fs)+1;
      trlend    = round((trg(i).time/1000+cfg.trialdef.poststim)*hdr.Fs)+1;
      trloffset = round(-cfg.trialdef.prestim*hdr.Fs);
      trl = [trl; trlbegin trlend trloffset];
    end
  end
  fprintf('%d triggers converted into %d trials\n', length(trg), size(trl,1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the average EEProbe file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif filetype(cfg.datafile, 'eep_avr')
  trl(1) = 1;
  trl(2) = hdr.nSamples;
  trl(3) = -hdr.nSamplesPre;
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the events in the BrainVision file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialdef') & isfield(cfg.trialdef, 'trgfile') & filetype(cfg.trialdef.trgfile, 'brainvision_vmrk')
  % read the external marker file in BrainVision format
  [stim, resp, segment, timezero] = read_brainvision_vmrk(cfg.trialdef.trgfile); 
  % set the default marker elements that can be converted into trials
  if ~isfield(cfg.trialdef, 'stim')
    cfg.trialdef.stim = [];
  end
  if ~isfield(cfg.trialdef, 'resp')
    cfg.trialdef.resp = [];
  end
  if ~isfield(cfg.trialdef, 'segment')
    cfg.trialdef.segment = 'no';
  end
  if ~isfield(cfg.trialdef, 'timezero')
    cfg.trialdef.timezero = 'no';
  end
  trl = [];
  for i=1:size(stim,1)
    if ismember(stim(i,1),cfg.trialdef.stim)
      trlbegin  = round((stim(i,2)/1000-cfg.trialdef.prestim)*hdr.Fs)+1;
      trlend    = round((stim(i,2)/1000+cfg.trialdef.poststim)*hdr.Fs)+1;
      trloffset = round(-cfg.trialdef.prestim*hdr.Fs);
      trl = [trl; trlbegin trlend trloffset];
     end
  end
  fprintf('%d stimulus markers converted into %d trials\n', size(stim,1), size(trl,1));
  tmp = size(trl,1);
  for i=1:size(resp,1)
    if ismember(resp(i,1),cfg.trialdef.resp)
      trlbegin  = round((resp(i,2)/1000-cfg.trialdef.prestim)*hdr.Fs)+1;
      trlend    = round((resp(i,2)/1000+cfg.trialdef.poststim)*hdr.Fs)+1;
      trloffset = round(-cfg.trialdef.prestim*hdr.Fs);
      trl = [trl; trlbegin trlend trloffset];
     end
  end
  fprintf('%d response markers converted into %d trials\n', size(resp,1), size(trl,1)-tmp);
  tmp = size(trl,1);
  if strcmp(cfg.trialdef.segment, 'yes')
    for i=1:size(segment,1)
      trlbegin  = round((segment(i)/1000-cfg.trialdef.prestim)*hdr.Fs)+1;
      trlend    = round((segment(i)/1000+cfg.trialdef.poststim)*hdr.Fs)+1;
      trloffset = round(-cfg.trialdef.prestim*hdr.Fs);
      trl = [trl; trlbegin trlend trloffset];
    end
  end
  fprintf('%d segment markers converted into %d trials\n', size(segment,1), size(trl,1)-tmp);
  tmp = size(trl,1);
  if strcmp(cfg.trialdef.timezero, 'yes')
    for i=1:size(timezero,1)
      trlbegin  = round((timezero(i)/1000-cfg.trialdef.prestim)*hdr.Fs)+1;
      trlend    = round((timezero(i)/1000+cfg.trialdef.poststim)*hdr.Fs)+1;
      trloffset = round(-cfg.trialdef.prestim*hdr.Fs);
      trl = [trl; trlbegin trlend trloffset];
    end
  end
  fprintf('%d "Time 0" markers converted into %d trials\n', size(timezero,1), size(trl,1)-tmp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the Neuromag file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif filetype(cfg.datafile, 'neuromag_fif')
  chanindx = match_str(hdr.label, cfg.trialdef.trgchan);
  nsamples = hdr.nTrials * hdr.nSamples;
  trigger = read_fcdc_data(cfg.datafile, hdr, 1, nsamples, chanindx, iscontinuous);
  trigindx = find(trigger & [0 diff(trigger)]);
  trl(:,1) = trigindx(:) - round(cfg.trialdef.prestim*hdr.Fs);  % define begin of each trial (in samples)
  trl(:,2) = trigindx(:) + round(cfg.trialdef.poststim*hdr.Fs); % define end of each trial (in samples)
  trl(:,3) = round(-cfg.trialdef.prestim*hdr.Fs);               % define the trial offset relative to latency zero (in samples) 
  trl(find(trl(:,1)<1), :) = [];                                % remove trials beginning before the start of the file
  trl(find(trl(:,2)>hdr.nTrials*hdr.nSamples), :) = [];         % remove trials after the end of the file
  fprintf('%d triggers converted into %d trials\n', length(trigindx), size(trl,1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the triggers or class-definitions in the epoched CTF file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialdef') & filetype(cfg.dataset, 'ctf_ds') & (isfield(cfg.trialdef, 'includeTrigger') | isfield(cfg.trialdef, 'excludeTrigger') | isfield(cfg.trialdef, 'includeConditions') | isfield(cfg.trialdef, 'excludeConditions'))
  % set empty defaults for the fields that do not exist
  if ~isfield(cfg.trialdef, 'includeTrigger'), cfg.trialdef.includeTrigger = []; end
  if ~isfield(cfg.trialdef, 'excludeTrigger'), cfg.trialdef.excludeTrigger = []; end
  if ~isfield(cfg.trialdef, 'includeConditions'), cfg.trialdef.includeConditions = {}; end
  if ~isfield(cfg.trialdef, 'excludeConditions'), cfg.trialdef.excludeConditions = {}; end
  % ensure that these are cell-arrays
  if ischar(cfg.trialdef.includeConditions), cfg.trialdef.includeConditions = {cfg.trialdef.includeConditions}; end
  if ischar(cfg.trialdef.excludeConditions), cfg.trialdef.excludeConditions = {cfg.trialdef.excludeConditions}; end  
  % Read trial definitions from the ClassFile.cls in the dataset
  [condNumbers,condLabels] = read_ctf_cls(fullfile(cfg.dataset, 'ClassFile.cls'));
  % Read trigger list from datafile
  triggerChan = read_ctf_trigger(cfg.dataset);
  % Shorten trials according to pre and poststim
  if isfield(cfg.trialdef,'prestim')
    nSamplesPreTmp = round(cfg.trialdef.prestim*hdr.Fs);
    nPreCorr = hdr.nSamplesPre - nSamplesPreTmp; 
    if nPreCorr < 0
      error('You have defined cfg.traildef.nsamplePre too large');
    end
  else
    nPreCorr = 0;
  end
  if isfield(cfg.trialdef,'poststim')
    nSamplesPostTmp = round(cfg.trialdef.poststim*hdr.Fs) ;
    nPostCorr = (hdr.nSamples-hdr.nSamplesPre) - nSamplesPostTmp; 
    if nPostCorr < 0
      error('You have defined cfg.traildef.nsamplePost too large');
    end
  else
    nPostCorr = 0;
  end
  % Extract the relevant trials from CTF data
  trl = [];
  l = 0; 
  for k=1:hdr.nTrials 
    if approveTrial(cfg,k,condNumbers,condLabels)
      % search 5 samples around where the trigger should be
      tbeg = (k-1)*hdr.nSamples+1 + hdr.nSamplesPre-2;
      tend = (k-1)*hdr.nSamples+1 + hdr.nSamplesPre+2;
      % be carefull not to to search outside of the data
      tbeg = max(tbeg, 1);
      tend = min(tend, hdr.nSamples*hdr.nTrials);
      % Pull out the trigger chan for trial k and find the trigger value,
      trigVal = max(triggerChan(tbeg:tend));
      if approveTrigger(cfg,trigVal) 
        l = l + 1; 
        trl(l,1) = 1+(k-1)*hdr.nSamples + nPreCorr;
        trl(l,2) = k*hdr.nSamples  - nPostCorr;
        trl(l,3) = -hdr.nSamplesPre + nPreCorr;
      end
    end
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the triggers in the continuous CTF file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialdef') & filetype(cfg.dataset, 'ctf_ds') & ~(isfield(cfg.trialdef, 'includeTrigger') | isfield(cfg.trialdef, 'excludeTrigger') | isfield(cfg.trialdef, 'includeConditions') | isfield(cfg.trialdef, 'excludeConditions') | isfield(cfg.trialdef, 'eventtype'))
  % read the triggers from the STIM channel
  fprintf('reading stimulus channel and detecting trigger onset\n');
  [backpanel, frontpanel] = read_ctf_trigger(cfg.dataset); 
  if isfield(cfg.trialdef, 'panel') 
    if strcmp(cfg.trialdef.panel, 'back')
      trigger = backpanel;
    elseif strcmp(cfg.trialdef.panel, 'front')
      trigger = frontpanel;
    elseif strcmp(cfg.trialdef.panel, 'both')
      % recombine the triggers on back and frontpanel into a single trigger value
      trigger = frontpanel + (2^16)*backpanel;
    else
      error('unrecognized option for cfg.trialdef.panel');
    end
  else
    % default is to use the backpanel where the Presentations PC is plugged into
    trigger = backpanel;
  end
  fprintf('total number of triggers is %d\n', length(find(trigger)));
  % find the samples at which the specified trigger occured
  trigindx = find(ismember(trigger, cfg.trialdef.trigger));
  % convert the trigger moments into trials
  trl(:,1) = trigindx(:) - round(cfg.trialdef.prestim*hdr.Fs);  % define begin of each trial (in samples)
  trl(:,2) = trigindx(:) + round(cfg.trialdef.poststim*hdr.Fs); % define end of each trial (in samples)
  trl(:,3) = round(-cfg.trialdef.prestim*hdr.Fs);               % define the trial offset relative to latency zero (in samples) 
  trl(find(trl(:,1)<1), :) = [];                                % remove trials beginning before the start of the file
  trl(find(trl(:,2)>hdr.nTrials*hdr.nSamples), :) = [];         % remove trials after the end of the file
  fprintf('%d triggers converted into %d trials\n', length(trigindx), size(trl,1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the epoched NeuroScan file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif filetype(cfg.datafile, 'ns_eeg')
  for k=1:hdr.nsweeps
    trl(k,1) = 1+(k-1)*hdr.npnt;
    trl(k,2) = k*hdr.npnt;
    trl(k,3) = (hdr.rate*hdr.xmin/1000);
  end
  fprintf('found %d trials in epoched neuroscan file\n', size(trl,1));
  fprintf('looking for manually accepted/rejected trials ');
  accept = ones(1,hdr.nsweeps);
  for k=1:hdr.nsweeps
    tmp = read_ns_eeg(cfg.datafile, k);
    accept(k) = tmp.sweep.accept;
    if accept(k)
      fprintf('.');
    else
      fprintf('R');
    end
  end
  fprintf('\n');
  trl = trl(find(accept), :);
  fprintf('accepted %d trials, rejected %d trials\n', sum(accept==1), sum(accept==0));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the trials from the events in the continuous NeuroScan file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif isfield(cfg, 'trialdef') & filetype(cfg.datafile, 'ns_cnt')
  % read the header and the event table from the continuous file
  tmp = read_ns_cnt(cfg.datafile, 'ldheaderonly', 1);
  fprintf('found %d events in continuous neuroscan file\n', length(tmp.event.frame));
  eventindx = find(ismember(tmp.event.stimtype, cfg.trialdef.trigger));
  trl(:,1) = tmp.event.frame(eventindx) - round(cfg.trialdef.prestim*tmp.rate) + 1;  % begin sample
  trl(:,2) = tmp.event.frame(eventindx) + round(cfg.trialdef.poststim*tmp.rate) + 1; % end sample   
  trl(:,3) = -round(cfg.trialdef.prestim*tmp.rate);                                  % offset
  fprintf('selected %d events based on triggercode\n', size(trl,1));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add the new trials and events to the output configuration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if exist('event', 'var')
  fprintf('found %d events\n', length(event));
  cfg.event = event;
else
  warning('no events were found');
end

if exist('trl', 'var')
  fprintf('created %d trials\n', size(trl,1));
  cfg.trl = trl;
else
  error('no trials were defined, see DEFINETRIAL for help');
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: definetrial.m,v 1.39 2005/02/16 15:18:25 roboos Exp $';

return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for single trial definition in case of CTF data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ok] = approveTrigger(cfg,trigVal)                        
ok = 1;
if isempty(cfg.trialdef.includeTrigger)                  
  ok = 1; 
else 
  ok = 0;
  for k=1:length(cfg.trialdef.includeTrigger)
    if trigVal == cfg.trialdef.includeTrigger(k)
      ok = 1; 
    end
  end 
end
if ~isempty(cfg.trialdef.excludeTrigger)                  
  for k=1:length(cfg.trialdef.excludeTrigger)
    if trigVal == cfg.trialdef.excludeTrigger(k)
      ok = 0; 
    end
  end 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION for single trial definition in case of CTF data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ok = approveTrial(cfg,Trial,condNumbers,condLabels)
if isempty(condLabels)
  ok = 1; 
  return
end
ok = 0;
if isfield(cfg.trialdef,'includeConditions') & ~isempty(cfg.trialdef.includeConditions)
  for k=1:length(cfg.trialdef.includeConditions) 
    for l=1:length(condLabels)
      if strcmp(char(condLabels{l}),char(cfg.trialdef.includeConditions{k})) & ismember(Trial,cell2mat(condNumbers{l}))
        fprintf('Including trial %d; Label: %s\n',Trial,char(cfg.trialdef.includeConditions{k}));
        ok = 1;
      end
    end
  end
else
  ok = 1;
end
if isfield(cfg.trialdef,'excludeConditions') & ~isempty(cfg.trialdef.excludeConditions)
  for k=1:length(cfg.trialdef.excludeConditions) 
    for l=1:length(condLabels)
      if strcmp(char(condLabels{l}),char(cfg.trialdef.excludeConditions{k})) & ismember(Trial,cell2mat(condNumbers{l}))
        fprintf('Excluding trial %d; Label: %s\n',Trial,char(cfg.trialdef.excludeConditions{k}));
        ok = 0;
      end
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that shows event table
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function show_event(event);
if isempty(event)
  fprintf('no events were found in the datafile\n');
  return
end  
eventtype = unique({event.type});
Neventtype = length(eventtype);
if Neventtype==0
  fprintf('no events were found in the datafile\n');
else
  fprintf('the following events were found in the datafile\n');
  for i=1:Neventtype
    sel = find(strcmp(eventtype{i}, {event.type}));
    try
      eventvalue = unique({event(sel).value});            % cell-array with string value
      eventvalue = sprintf('''%s'' ', eventvalue{:});     % translate into a single string
    catch
      eventvalue = unique(cell2mat({event(sel).value}));  % array with numeric values or empty
      eventvalue = num2str(eventvalue);                   % translate into a single string
    end
    fprintf('event type: ''%s'' ', eventtype{i});
    fprintf('with event values: %s', eventvalue);
    fprintf('\n');
  end
end
