function [down] = downsamplevolume(cfg, source);

% DOWNSAMPLEVOLUME downsamples an anatomical MRI or source reconstruction
% and optionally normalizes its coordinate axes, keeping the homogenous
% transformation matrix correct.
%
% Use as
%   [source] = downsamplevolume(cfg, source)
% where the cconfiguration can contain
%   cfg.downsample    = integer number, default 1 (no downsampling)
%   cfg.voxelcoord    = 'yes' or 'no'
%
% If you specify "voxelcoord", the coordinate axes as defined by
% xgrid/ygrid/zgrid will be changed to 1:Nxgrid, 1:Nygrid and 1:Nzgrid.
% The default behaviour is to retain the coordinate axes as defined in
% the original volume.
%
% This function is used by SOUREINTERPOLATE, SOURCEREAD and SOURCENORMALIZE.

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: downsamplevolume.m,v $
% Revision 1.3  2005/02/16 15:10:05  roboos
% added new volume fields from pcc beamformer
%
% Revision 1.2  2005/02/08 08:44:38  roboos
% fixed error in computation of new homogenous coordinate transformation matrix (in case of voxelcoords=yes), moved code to separate function (transformgrid)
% replaced computation for all scalar volumes by a for-loop over a parameter list
% implemented downsampling of inside/outside field in same way as the other volumes (keepinside=yes|no)
%
% Revision 1.1  2004/09/08 12:38:33  jansch
% Inserted version information, and moved function out of private folder. The
% function is the same to /private/downsamplevolume.m in older versions of the
% toolbox.
%
% Revision 1.2  2004/08/26 11:12:56  roboos
% added support for all known source parameters, updated help
%
% Revision 1.1  2004/08/26 09:12:26  roboos
% new implementation, to be used by different sourceXXX functions
%

if ~isfield(cfg, 'downsample'), cfg.downsample = 1;     end
if ~isfield(cfg, 'keepinside'), cfg.keepinside = 'yes'; end

if ~isfield(cfg, 'voxelcoord'),
  % the default depends on the input
  if all(source.xgrid==1:length(source.xgrid)) & ...
     all(source.ygrid==1:length(source.ygrid)) & ...
     all(source.zgrid==1:length(source.zgrid))
    cfg.voxelcoord = 'yes';	% keep it in voxel coordinates
  else
    cfg.voxelcoord = 'no';	% keep it in head coordinates
  end
end

% ensure that it has a valid transformation matrix
if ~isfield(source, 'transform')
  % assume that the position of each volume element is described properly with xgrid/ygrid/zgrid
  source.transform = eye(4);
end

% select the voxels that will be kept in the downsampled output volume
xsel = 1:cfg.downsample:source.dim(1);
ysel = 1:cfg.downsample:source.dim(2);
zsel = 1:cfg.downsample:source.dim(3);
down.dim = [length(xsel) length(ysel) length(zsel)];

if strcmp(cfg.voxelcoord, 'no')
  % keep the original coordinate transformation and downsampled axes definition
  down.transform = source.transform;
  down.xgrid     = source.xgrid(xsel);
  down.ygrid     = source.ygrid(ysel);
  down.zgrid     = source.zgrid(zsel);
else
  % optionally change the coordinate axes to voxel coordinates
  [down.xgrid, down.ygrid, down.zgrid, down.transform] = transformgrid(source.xgrid(xsel), source.ygrid(ysel), source.zgrid(zsel), source.transform);
end

if strcmp(cfg.keepinside, 'yes') & isfield(source, 'inside') & isfield(source, 'outside')
  % reformat the inside field to a full volume so that it can be downsampled as well
  tmp = [];
  tmp(source.inside)  = 1;
  tmp(source.outside) = 0;
  source.inside = tmp;
else
  cfg.keepinside = 'no';
  % remove the inside and outside fields to prevent them from being downsampled
  if isfield(source, 'inside' ), source = rmfield(source, 'inside' ); end
  if isfield(source, 'outside'), source = rmfield(source, 'outside'); end
end

% define the list of known volume parameters
param = {
  'inside'
  'anatomy'
  'mask'
  'zscore'
  'tscore'
  'prob'  
  'avg.pow'
  'avg.coh'
  'avg.nai'
  'avg.noise'  
  'avg.mom'  
  'avg.csd'  
  'avg.cov'  
  'avg.noisecsd'  
  'avg.noisecov'  
  'var.pow'
  'var.coh'
  'var.nai'
  'var.noise'
  'sem.pow'
  'sem.coh'
  'sem.nai'
  'sem.noise'
  'avgA.pow'
  'avgA.coh'
  'avgA.nai'
  'avgA.noise'
  'avgB.pow'
  'avgB.coh'
  'avgB.nai'
  'avgB.noise'
};

% downsample each of the known scalar parameters that represents a volume
for i=1:length(param)
  if issubfield(source, param{i})
    fprintf('downsampling %s\n', param{i});
    tmp  = reshape(getsubfield(source, param{i}), source.dim);    % ensure that it is a 3D volume
    down = setsubfield(down, param{i}, tmp(xsel, ysel, zsel));    % downsample the volume
  end
end

if strcmp(cfg.keepinside, 'yes')
  % restore the inside and outside fields in their proper formats
  tmp          = down.inside;
  down.inside  = find(tmp==1);
  down.outside = find(tmp~=1);
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: downsamplevolume.m,v 1.3 2005/02/16 15:10:05 roboos Exp $';

% remember the configuration of the input objects
if isfield(source,'cfg'), cfg.previous = source.cfg ; end

% remember the exact configuration details
down.cfg = cfg;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the following code snippet is for debugging only
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% select twp different voxels in the original MRI and compute their headcoordinates
% p1 = [source.xgrid(1) source.ygrid(1) source.zgrid(1) 1];
% p3 = [source.xgrid(1+1*cfg.downsample) source.ygrid(1+2*cfg.downsample) source.zgrid(1+3*cfg.downsample) 1];
% (source.transform * p1')'
% (source.transform * p3')'
% 
% select the same voxels in the downsampled MRI and compute their headcoordinates, they should be the same
% p1 = [down.xgrid(1) down.ygrid(1) down.zgrid(1) 1];
% p3 = [down.xgrid(1+1) down.ygrid(1+2) down.zgrid(1+3) 1];
% (down.transform * p1')'
% (down.transform * p3')'
