function [normalise] = normalisevolume(cfg, interp)

% NORMALISEVOLUME normalizes anatomical and functional volume data to a template.
% This function uses the SPM2-toolbox, see http://www.fil.ion.ucl.ac.uk/spm/
%
% Use as
%   [normalise] = normalisevolume(cfg, interp)
%
% The input is the result from SOURCEINTERPOLATE. Alternatively, the input
% data can contain a single anatomical MRI that was read with READ_FCDC_MRI,
% or you can specify a filename of an anatomical MRI.
%
% Configuration options are:
%   cfg.template    = '/opt/spm2/templates/T1.mnc' or string
%   cfg.parameter   = cell-array with the functional data which has to
%                     be normalised, can be 'all'
%   cfg.downsample  = number (default = 1, meaning no downsampling)
%   cfg.name        = string for output filename
%   cfg.write       = 'no' (default) or 'yes', writes the segmented volumes to SPM2
%                     compatible analyze-file, with the suffix
%                     _anatomy for the anatomical MRI volume
%                     _param   for each of the functional volumes

% Undocumented options
%   cfg.keepintermediate = 'yes' or 'no'
%   cfg.intermediatename = prefix of the the coregistered images and of the
%                          original images in the original headcoordinate system

% Copyright (C) 2004, Jan-Mathijs Schoffelen
%
% $Log: normalisevolume.m,v $
% Revision 1.1  2005/03/03 10:44:38  roboos
% Cleaned up the sourcenormalise function and moved everything to a
% new function with the name normalisevolume.  The new function has a
% different way of handling the subfields that can be normalised, and uses
% a different (and more robust) initial pre-alignment of the input volume
% with the SPM template. Also the type-detection (i.e. either SPM or CTF)
% of the input volume has been changed.
%
% Revision 1.6  2005/02/09 13:08:55  roboos
% changed the order in which the parameters were assigned, there was a probloem if cfg.parameter was not 'all'
%
% Revision 1.5  2005/02/08 14:04:45  roboos
% implemented keepinside, changed handling of parameters to use getsubfield/issubfield
% separated default transformation as a separate variable (no functional change yet)
%
% Revision 1.4  2004/09/10 11:47:01  jansch
% fixed bug in checking for the presence of specified parameters in
% configuration
%
% Revision 1.3  2004/09/09 05:48:13  jansch
% removed bug using an ambiguous function for the rotation of the transform
% matrix.
%
% Revision 1.2  2004/09/08 11:15:04  jansch
% added version-information to output.
%

% check if spm2 is in your path, see http://www.fil.ion.ucl.ac.uk/spm
hasspm = (exist('spm_vol') & exist('spm_write_vol') & exist('spm_normalise'));
if ~hasspm
  error('the SPM2 toolbox is not installed, see: http://www.fil.ion.ucl.ac.uk/spm/');
end

% set the defaults
if ~isfield(cfg,'template'),         cfg.template = '/opt/spm2/templates/T1.mnc'; end;
if ~isfield(cfg,'parameter'),        cfg.parameter = 'all';                       end;
if ~isfield(cfg,'downsample'),       cfg.downsample = 1;                          end;
if ~isfield(cfg,'write'),            cfg.write = 'no';                            end;
if ~isfield(cfg,'keepinside'),       cfg.keepinside = 'yes';                      end;
if ~isfield(cfg,'keepintermediate'), cfg.keepintermediate = 'no';                 end;
if ~isfield(cfg,'coordinates'),      cfg.coordinates = [];                        end;

if ~isfield(cfg,'intermediatename')
  cfg.intermediatename = tempname;
end

if ~isfield(cfg,'name') & strcmp(cfg.write,'yes')
  error('you must specify the output filename in cfg.name');
end

if isempty(cfg.template),
  error('you must specify a template anatomical MRI');
end

if ischar(interp),
  % read the anatomical MRI data from file
  filename = interp;
  fprintf('reading MRI from file\n');
  interp = read_fcdc_mri(filename);
  if filetype(filename, 'ctf_mri') & isempty(cfg.coordinates)
    % based on the filetype assume that the coordinates correspond with CTF convention
    cfg.coordinates = 'ctf';
  end
end

if ~isfield(interp,'anatomy'),
  error('no anatomical information available, this is required for normalisation');
end

% downsample the volume, this also ensures that the homogenous
% transformation towards headcoordinates is defined for xgrid=1:Nx etc,
% and not from an arbitrary definition of xgrid/ygrid/zgrid
tmpcfg = [];
tmpcfg.downsample = cfg.downsample;
tmpcfg.voxelcoords = 'yes';
interp = downsamplevolume(tmpcfg, interp);

% remember the original transformation matrix
original.transform = interp.transform;
original.nosedir = [];
original.origin = [];

if isempty(cfg.coordinates)
  % determine in which direction the nose is pointing
  while isempty(original.nosedir)
    response = input('Is the nose pointing in the positive X- or Y-direction? [x/y] ','s');
    if strcmp(response, 'x'),
      original.nosedir = 'positivex';
    elseif strcmp(response, 'y'),
      original.nosedir = 'positivey';
    end
  end

  % determine where the origin is
  while isempty(original.origin)
    response = input('Is the origin on the Anterior commissure or between the Ears? [a/e] ','s');
    if strcmp(response, 'e'),
      original.origin = 'interauricular';
    elseif strcmp(response, 'a'),
      original.origin = 'ant_comm';
    end
  end

  % determine the coordinatesystem of the input MRI volume
  if strcmp(original.origin, 'interauricular') & strcmp(original.nosedir, 'positivex')
    cfg.coordinates = 'ctf';
  elseif strcmp(original.origin, 'ant_comm') & strcmp(original.nosedir, 'positivey')
    cfg.coordinates = 'spm';
  end
end

% ensure that the input MRI volume is approximately aligned with the SPM template
if strcmp(cfg.coordinates, 'ctf')
  fprintf('assuming CTF coordinates for input, i.e. positive X-axis towards nasion and Y-axis through ears\n');
  % flip, rotate and translate the CTF mri so that it approximately corresponds with SPM coordinates
  interp = align_ctf2spm(interp);
elseif strcmp(cfg.coordinates, 'spm')
  fprintf('assuming that the input MRI is already approximately alligned with SPM coordinates\n');
  % nothing needs to be done
else
  error('cannot determine the (approximate) alignmenmt of the input MRI with the SPM template');
end

warning off;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine the parameters that should be normalised
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define the list of known volume parameters
% the anatomy and inside volume are treated separately
parameters = {
  'mask'
  'zscore'
  'tscore'
  'prob'
  'avg.pow'
  'avg.coh'
  'avg.nai'
  'avg.noise'
%  'avg.mom'  
%  'avg.csd'  
%  'avg.cov'  
%  'avg.noisecsd'  
%  'avg.noisecov'  
  'var.pow'
  'var.coh'
  'var.nai'
  'var.noise'
  'sem.pow'
  'sem.coh'
  'sem.nai'
  'sem.noise'
  'avgA.pow'
  'avgA.coh'
  'avgA.nai'
  'avgA.noise'
  'avgB.pow'
  'avgB.coh'
  'avgB.nai'
  'avgB.noise'
};

if strcmp(cfg.keepinside, 'yes') & isfield(interp, 'inside') & isfield(interp, 'outside')
  % reformat the inside field to a full volume so that it can be downsampled as well
  tmp = [];
  tmp(interp.inside)  = 1;
  tmp(interp.outside) = 0;
  interp.inside  = tmp;
  interp.outside = [];
else
  cfg.keepinside = 'no';
  % remove the inside and outside fields to prevent them from being downsampled
  if isfield(interp, 'inside' ), interp = rmfield(interp, 'inside' ); end
  if isfield(interp, 'outside'), interp = rmfield(interp, 'outside'); end
end

% the inside should be normalized if requested
if strcmp(cfg.keepinside, 'yes')
  parameters = {'inside' parameters{:}};
end

% select only the parameters that are present as volumes in the input
sel = [];
for i=1:length(parameters)
  if issubfield(interp, parameters{i})
    sel = [sel; i];
  end
end
parameters = parameters(sel);

if strcmp(cfg.parameter,'all')
  % keep all parameters that are present in the source reconstruction
  parameters = parameters;
else
  % select only the parameters specified in the configuration
  parameters = intersect(cfg.parameter, parameters);
end

% the anatomy should always be normalized as the first volume
parameters = {'anatomy' parameters{:}};

% remember this for the output configuration
cfg.parameter = parameters;

% ensure that all parameters are present as 3D volumes
for parlop=1:length(parameters)
  interp = setsubfield(interp, parameters{parlop}, reshape(getsubfield(interp, parameters{parlop}), interp.dim));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% here the normalisation starts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% create an spm-compatible header for the anatomical volume data
Va = writevolume(interp, 'anatomy', [cfg.intermediatename,'_anatomy.img']);

% create an spm-compatible file for each of the functional volumes
for parlop = 2:length(parameters)  % skip the anatomy
  tmp = parameters{parlop};
  tmp(find(tmp=='.')) = '_';
  writevolume(interp, parameters{parlop}, [cfg.intermediatename '_' tmp '.img']);
end

% read the template anatomical volume data
if strcmp(char(cfg.template(end-2:end)),'mnc'),
  Vtempl    = spm_vol_minc(cfg.template);
elseif strcmp(char(cfg.template(end-2:end)),'img'),
  Vtempl    = spm_vol(cfg.template);
else
  error('Unknown template');
end

fprintf('performing the normalisation\n');
% do spatial normalisation according to these steps
% step 1: read header information for template and source image
% step 2: compute transformation parameters
% step 3: write the results to a file with prefix 'w'

Va        = spm_vol([cfg.intermediatename,'_anatomy.img']);
params    = spm_normalise(Vtempl,Va);
flags.vox = [cfg.downsample,cfg.downsample,cfg.downsample];
files     = {};

% remember the normalisation parameters in the configuration
cfg.spmparams = params;

% apply the normalisation parameters to each of the volumes
for parlop=1:length(parameters)
  fprintf('creating normalised analyze-file for %s\n', parameters{parlop});
  files{parlop} = sprintf('%s_%s.img', cfg.intermediatename, parameters{parlop});
  [p, f, x] = fileparts(files{parlop});
  wfiles{parlop} = fullfile(p, ['w' f x]);
end
spm_write_sn(char(files),params,flags);  % this creates the 'w' prefixed files

normalise = [];

% read the normalised results from the 'w' prefixed files
V = spm_vol(char(wfiles));
for vlop=1:length(V)
  normalise = setsubfield(normalise, parameters{vlop}, spm_read_vols(V(vlop)));
end

normalise.transform = V(1).mat;
normalise.dim       = size(normalise.anatomy);
normalise.xgrid     = 1:normalise.dim(1);
normalise.ygrid     = 1:normalise.dim(2);
normalise.zgrid     = 1:normalise.dim(3);

if strcmp(cfg.keepinside, 'yes')
  % restore the inside and outside fields in their proper formats
  tmp               = normalise.inside(:);
  normalise.inside  = find(abs(tmp-1)<=10*eps);
  normalise.outside = setdiff(1:length(tmp), normalise.inside)';
end

% flip and permute the dimensions to align the volume with the headcoordinate axes
normalise = align_ijk2xyz(normalise);

if strcmp(cfg.write,'yes')
  % create an spm-compatible file for each of the normalised volumes
  for parlop=1:length(parameters)  % include the anatomy
    tmp = parameters{parlop};
    tmp(find(tmp=='.')) = '_';
    writevolume(normalise, parameters{parlop}, [cfg.name '_' tmp '.img']);
  end
end

if strcmp(cfg.keepintermediate,'no')
  % remove the intermediate files
  for flop=1:length(files)
    [p, f, x] = fileparts(files{flop});
    delete(fullfile(p, [f, '.*']));
    [p, f, x] = fileparts(wfiles{flop});
    delete(fullfile(p, [f, '.*']));
  end
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: normalisevolume.m,v 1.1 2005/03/03 10:44:38 roboos Exp $';

% remember the configuration of the input objects
if isfield(interp,'cfg'), cfg.previous = interp.cfg ; end

% remember the exact configuration details
normalise.cfg = cfg;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that writes a volume to an SPM compatible file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Va] = writevolume(interp, param, filename);
% create an spm-compatible header for the anatomical volume data
fprintf('creating analyze-file for %s\n', param);
dat = getsubfield(interp, param);
typ = spm_type(class(dat));
Va         = [];
Va.mat     = interp.transform;
Va.fname   = filename;
Va.dim     = [interp.dim typ];
Va.n       = 1;
Va.pinfo   = [1;0;0];
Va.private.hdr.dime.datatype = typ;
Va = spm_create_vol(Va);
Va = spm_write_vol(Va,dat);

