function [filetype, detail] = filetype(filename, desired);

% FILETYPE determines the filetype of many EEG/MEG/MRI data files
% by looking at the name, extension and optionally (part of) its
% contents.
%
% It tries to determine the global type of file (which usually
% corresponds to the manufacturer of the recording system) and the
% particular subtype (e.g. continuous, average).
%
% You can use this function either as
%   [type, detail] = filetype(filename)
% to get a descriptive string with the filetype, or as
%   [bool] = filetype(filename, type)
% to get a boolean value indicating whether the file is of the
% desired type.
%
% The filetypes of the following manufacturers are recognized
%  - CTF
%  - Neuromag
%  - ASA
%  - EDF
%  - Neuroscan
%  - Curry
%  - Analyse
%  - EEProbe
%  - BrainVision
%  - BESA (very small subset)
%  - MINC
%  - AFNI
%
% See also READ_XXX_YYY where XXX=manufacturer and YYY=subtype

% Copyright 2003 (C) Robert Oostenveld
%
% $Log: filetype.m,v $
% Revision 1.10  2005/02/16 08:06:32  roboos
% changed behavior of magic number check: if file cannot be read, give warning (instead of error) and indicate false
% changed indentation
%
% Revision 1.9  2005/02/11 07:53:35  roboos
% iadded support for Curry, Polhemus, Plexon and  Besa *.eps
%
% Revision 1.8  2005/02/07 19:26:19  roboos
% added Ole Jensen's *.lay fileformat
%
% Revision 1.7  2004/10/28 09:42:19  roboos
% added AFNI brick and head (based on extension)
%
% Revision 1.6  2004/08/26 15:57:09  roboos
% changed file extension check into case insensitive
% added support for brainvision_marker
%
% Revision 1.5  2004/08/26 11:35:30  roboos
% added MINC based on mnc extension
%
% Revision 1.4  2004/03/29 15:14:30  roberto
% added support for *.dat as brainvision_dat (exported from BVA)
%
% Revision 1.3  2004/03/23 14:46:13  roberto
% fixed bug in eeprobe average, made besa average smarter
%
% Revision 1.2  2004/03/10 14:06:41  roberto
% added Neuromag filetypes, updated and improved help
%
% Revision 1.1  2003/12/05 10:59:43  roberto
% initial submission
%

if nargin<2
  desired = [];
end

% start with unknown values
filetype     = 'unknown';
manufacturer = 'unknown';
content      = 'unknown';

% known CTF file types
if check_extension(filename, '.ds') & isdir(filename)
  filetype = 'ctf_ds';
  manufacturer = 'CTF';
  content = 'MEG dataset';
elseif check_extension(filename, '.res4') & (check_header(filename, 'MEG41RS') | check_header(filename, 'MEG4RES'))
  filetype = 'ctf_res4';
  manufacturer = 'CTF';
  content = 'MEG/EEG header information';
elseif check_extension(filename, '.meg4') & check_header(filename, 'MEG41CP')
  filetype = 'ctf_meg4';
  manufacturer = 'CTF';
  content = 'MEG/EEG';
elseif check_extension(filename, '.mrk') & check_header(filename, 'PATH OF DATASET:')
  filetype = 'ctf_mrk';
  manufacturer = 'CTF';
  content = 'marker file';
elseif check_extension(filename, '.mri') & check_header(filename, 'CTF_MRI_FORMAT VER 2.2')
  filetype = 'ctf_mri';
  manufacturer = 'CTF';
  content = 'MRI';
elseif check_extension(filename, '.hdm')
  filetype = 'ctf_hdm';
  manufacturer = 'CTF';
  content = 'volume conduction model';
elseif check_extension(filename, '.hc')
  filetype = 'ctf_hc';
  manufacturer = 'CTF';
  content = 'headcoil locations';
elseif check_extension(filename, '.shape')
  filetype = 'ctf_shape';
  manufacturer = 'CTF';
  content = 'headshape points';
elseif check_extension(filename, '.shape_info')
  filetype = 'ctf_shapeinfo';
  manufacturer = 'CTF';
  content = 'headshape information';

  % known EEProbe file types
elseif check_extension(filename, '.cnt') & check_header(filename, 'RIFF')
  filetype = 'eep_cnt';
  manufacturer = 'EEProbe';
  content = 'EEG';
elseif check_extension(filename, '.avr') & check_header(filename, char([38 0 16 0]))
  filetype = 'eep_avr';
  manufacturer = 'EEProbe';
  content = 'ERP';
elseif check_extension(filename, '.trg')
  filetype = 'eep_trg';
  manufacturer = 'EEProbe';
  content = 'trigger information';
elseif check_extension(filename, '.rej')
  filetype = 'eep_rej';
  manufacturer = 'EEProbe';
  content = 'rejection marks';

  % known ASA file types
elseif check_extension(filename, '.elc')
  filetype = 'asa_elc';
  manufacturer = 'ASA';
  content = 'electrode positions';
elseif check_extension(filename, '.vol')
  filetype = 'asa_vol';
  manufacturer = 'ASA';
  content = 'volume conduction model';
elseif check_extension(filename, '.bnd')
  filetype = 'asa_bnd';
  manufacturer = 'ASA';
  content = 'boundary element model details';
elseif check_extension(filename, '.msm')
  filetype = 'asa_msm';
  manufacturer = 'ASA';
  content = 'ERP';
elseif check_extension(filename, '.msr')
  filetype = 'asa_msr';
  manufacturer = 'ASA';
  content = 'ERP';
elseif check_extension(filename, '.dip')
  % FIXME, can also be CTF dipole file
  filetype = 'asa_dip';
  manufacturer = 'ASA';
elseif check_extension(filename, '.mri')
  % FIXME, can also be CTF mri file
  filetype = 'asa_mri';
  manufacturer = 'ASA';
  content = 'MRI image header';
elseif check_extension(filename, '.iso')
  filetype = 'asa_iso';
  manufacturer = 'ASA';
  content = 'MRI image data';

  % known Neuroscan file types
elseif check_extension(filename, '.avg') & check_header(filename, 'Version 3.0')
  filetype = 'ns_avg';
  manufacturer = 'Neuroscan';
  content = 'averaged EEG';
elseif check_extension(filename, '.cnt') & check_header(filename, 'Version 3.0')
  filetype = 'ns_cnt';
  manufacturer = 'Neuroscan';
  content = 'continuous EEG';
elseif check_extension(filename, '.eeg') & check_header(filename, 'Version 3.0')
  filetype = 'ns_eeg';
  manufacturer = 'Neuroscan';
  content = 'epoched EEG';

  % known Analyze file types
elseif check_extension(filename, '.hdr')
  filetype = 'analyze_hdr';
  manufacturer = 'Mayo Analyze';
  content = 'PET/MRI image header';
elseif check_extension(filename, '.img')
  filetype = 'analyze_img';
  manufacturer = 'Mayo Analyze';
  content = 'PET/MRI image data';

elseif check_extension(filename, '.mnc')
  filetype = 'minc';
  content = 'MRI image data';

elseif check_extension(filename, '.brik') | check_extension(filename, '.BRIK')
  filetype = 'afni_brik';
  content = 'MRI image data';
elseif check_extension(filename, '.head') | check_extension(filename, '.HEAD')
  filetype = 'afni_head';
  content = 'MRI header data';

  % known BrainVison file types
elseif check_extension(filename, '.vhdr')
  filetype = 'brainvision_vhdr';
  manufacturer = 'BrainProducts';
  content = 'EEG header';
elseif check_extension(filename, '.vmrk')
  filetype = 'brainvision_vmrk';
  manufacturer = 'BrainProducts';
  content = 'EEG markers';
elseif check_extension(filename, '.vabs')
  filetype = 'brainvision_vabs';
  manufacturer = 'BrainProducts';
  content = 'Brain Vison Analyzer macro';
elseif check_extension(filename, '.eeg')
  % FIXME, can also be Neuroscan epoched EEG data
  filetype = 'brainvision_eeg';
  manufacturer = 'BrainProducts';
  content = 'continuous EEG data';
elseif check_extension(filename, '.seg')
  filetype = 'brainvision_seg';
  manufacturer = 'BrainProducts';
  content = 'segmented EEG data';
elseif check_extension(filename, '.dat')
  % WARNING this is a very general name, it could be exported BrainVision data
  filetype = 'brainvision_dat';
  manufacturer = 'BrainProducts';
  content = 'exported EEG data';
elseif check_extension(filename, '.marker')
  filetype = 'brainvision_marker';
  manufacturer = 'BrainProducts';
  content = 'rejection markers';

  % known Polhemus file types
elseif check_extension(filename, '.pos')
  filetype = 'polhemus_pos';
  manufacturer = 'BrainProducts/CTF/Polhemus?'; % actually I don't know whose software it is
  content = 'electrode positions';

  % known Plextor file types
elseif check_extension(filename, '.nex')  & check_header(filename, 'NEX1')
  filetype = 'plextor_nex';
  manufacturer = 'Plextor';
  content = 'electrophysiological data';
elseif check_extension(filename, '.plx')  & check_header(filename, 'PLEX')
  filetype = 'plextor_nex';
  manufacturer = 'Plextor';
  content = 'electrophysiological data';
elseif check_extension(filename, '.ddt')
  filetype = 'plextor_nex';
  manufacturer = 'Plextor';

  % known Neuromag file types
elseif check_extension(filename, '.fif')
  filetype = 'neuromag_fif';
  manufacturer = 'Neuromag';
  content = 'MEG header and data';
elseif check_extension(filename, '.bdip')
  filetype = 'neuromag_bdip';
  manufacturer = 'Neuromag';
  content = 'dipole model';

  % known BESA file types
elseif check_extension(filename, '.avr') & strcmp(filetype, 'unknown')
  filetype = 'besa_avr';  % FIXME, can also be EEProbe average EEG
  manufacturer = 'BESA';
  content = 'average EEG';
elseif check_extension(filename, '.elp')
  filetype = 'besa_elp';
  manufacturer = 'BESA';
  content = 'electrode positions';
elseif check_extension(filename, '.eps')
  filetype = 'besa_eps';
  manufacturer = 'BESA';
  content = 'digitizer information';
elseif check_extension(filename, '.sfp')
  filetype = 'besa_sfp';
  manufacturer = 'BESA';
  content = 'sensor positions';
elseif check_extension(filename, '.ela')
  filetype = 'besa_ela';
  manufacturer = 'BESA';
  content = 'sensor information';

  % known Curry V4 file types
elseif check_extension(filename, '.dap')
  filetype = 'curry_dap';
  manufacturer = 'Curry';
  content = 'data parameter file';
elseif check_extension(filename, '.dat')
  filetype = 'curry_dat';
  manufacturer = 'Curry';
  content = 'raw data file';
elseif check_extension(filename, '.rs4')
  filetype = 'curry_rs4';
  manufacturer = 'Curry';
  content = 'sensor geometry file';
elseif check_extension(filename, '.par')
  filetype = 'curry_par';
  manufacturer = 'Curry';
  content = 'data or image parameter file';
elseif check_extension(filename, '.bd0') | check_extension(filename, '.bd1') | check_extension(filename, '.bd2') | check_extension(filename, '.bd3') | check_extension(filename, '.bd4') | check_extension(filename, '.bd5') | check_extension(filename, '.bd6') | check_extension(filename, '.bd7') | check_extension(filename, '.bd8') | check_extension(filename, '.bd9')
  filetype = 'curry_bd';
  manufacturer = 'Curry';
  content = 'BEM description file';
elseif check_extension(filename, '.bt0') | check_extension(filename, '.bt1') | check_extension(filename, '.bt2') | check_extension(filename, '.bt3') | check_extension(filename, '.bt4') | check_extension(filename, '.bt5') | check_extension(filename, '.bt6') | check_extension(filename, '.bt7') | check_extension(filename, '.bt8') | check_extension(filename, '.bt9')
  filetype = 'curry_bt';
  manufacturer = 'Curry';
  content = 'BEM transfer matrix file';
elseif check_extension(filename, '.bm0') | check_extension(filename, '.bm1') | check_extension(filename, '.bm2') | check_extension(filename, '.bm3') | check_extension(filename, '.bm4') | check_extension(filename, '.bm5') | check_extension(filename, '.bm6') | check_extension(filename, '.bm7') | check_extension(filename, '.bm8') | check_extension(filename, '.bm9')
  filetype = 'curry_bm';
  manufacturer = 'Curry';
  content = 'BEM full matrix file';
elseif check_extension(filename, '.dig')
  filetype = 'curry_dig';
  manufacturer = 'Curry';
  content = 'digitizer file';

  % known Curry V2 file types
elseif check_extension(filename, '.sp0') | check_extension(filename, '.sp1') | check_extension(filename, '.sp2') | check_extension(filename, '.sp3') | check_extension(filename, '.sp4') | check_extension(filename, '.sp5') | check_extension(filename, '.sp6') | check_extension(filename, '.sp7') | check_extension(filename, '.sp8') | check_extension(filename, '.sp9')
  filetype = 'curry_sp';
  manufacturer = 'Curry';
  content = 'point list';
elseif check_extension(filename, '.s10') | check_extension(filename, '.s11') | check_extension(filename, '.s12') | check_extension(filename, '.s13') | check_extension(filename, '.s14') | check_extension(filename, '.s15') | check_extension(filename, '.s16') | check_extension(filename, '.s17') | check_extension(filename, '.s18') | check_extension(filename, '.s19') | check_extension(filename, '.s20') | check_extension(filename, '.s21') | check_extension(filename, '.s22') | check_extension(filename, '.s23') | check_extension(filename, '.s24') | check_extension(filename, '.s25') | check_extension(filename, '.s26') | check_extension(filename, '.s27') | check_extension(filename, '.s28') | check_extension(filename, '.s29') | check_extension(filename, '.s30') | check_extension(filename, '.s31') | check_extension(filename, '.s32') | check_extension(filename, '.s33') | check_extension(filename, '.s34') | check_extension(filename, '.s35') | check_extension(filename, '.s36') | check_extension(filename, '.s37') | check_extension(filename, '.s38') | check_extension(filename, '.s39')
  filetype = 'curry_s';
  manufacturer = 'Curry';
  content = 'triangle or tetraedra list';
elseif check_extension(filename, '.pom')
  filetype = 'curry_pom';
  manufacturer = 'Curry';
  content = 'anatomical localization file';
elseif check_extension(filename, '.res')
  filetype = 'curry_res';
  manufacturer = 'Curry';
  content = 'functional localization file';

  % some other known file types
elseif check_extension(filename, '.lay')
  filetype = 'layout';
  manufacturer = 'Ole Jensen';
  content = 'layout of channels for plotting';
elseif check_extension(filename, '.dcm')
  filetype = 'dicom';
  manufacturer = 'Dicom';
  content = 'image data';
elseif check_extension(filename, '.trl')
  filetype = 'fcdc_trl';
  manufacturer = 'F.C.Donders';
  content = 'trial definitions';
elseif check_extension(filename, '.edf')
  filetype = 'edf';
  manufacturer = 'European Data Format';
  content = 'electrophysiological data';
elseif check_extension(filename, '.mat')
  filetype = 'matlab';
  manufacturer = 'Matlab';
  content = 'Matlab binary data';
end

if strcmp(filetype, 'unknown')
  warning(sprintf('could not determine filetype of %s', filename));
end

% remember the details
detail.filetype     = filetype;
detail.manufacturer = manufacturer;
detail.content      = content;

if ~isempty(desired)
  % return a boolean value instead of a descriptive string
  filetype = strcmp(filetype, desired);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val] = check_extension(filename, ext)
% perform case insensitive string comparison of the extension
if length(filename)<length(ext)
  val = 0;
else
  val = strcmpi(filename((end-length(ext)+1):end), ext);
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val] = check_header(filename, head)
fid = fopen(filename, 'rb');
if fid<0
  warning(sprintf('could not open %s', filename));
  val = 0;
else
  [str, siz] = fread(fid, length(head), 'char');
  fclose(fid);
  if siz~=length(head)
    error('could not read the header from the file');
  end
  val = all(char(str(:))==head(:));
end
return

