function [pnt] = prepare_brain_surface(cfg, grad, vol);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that makes a brain surface from the headhape and places a lot
% of dipoles on it. The dipoles subsequently can be used for a simplified 
% distributed source model.
%
% This function uses the following fields from the configuration
%   cfg.headshape
%   cfg.spheremesh
%   cfg.inwardshift
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Copyright (C) 2004, Robet Oostenveld
%
% $Log: prepare_brain_surface.m,v $
% Revision 1.1  2004/06/28 08:59:38  roboos
% moved files from fieldtrip to fieldtrip/private
%
% Revision 1.3  2004/05/17 07:22:16  roberto
% fixed bug in triangle orientation for multi-sphere
%
% Revision 1.2  2004/04/26 12:36:49  roberto
% fixed bug in case of multisphere head surface
%
% Revision 1.1  2004/04/08 15:51:20  roberto
% initial submissiion into cvs
%

if strcmp(cfg.headshape, 'headmodel')
  if length(vol.r)==1
    fprintf('constructing brain surface from single sphere model\n');
    % single sphere model, use a refined icosahedron 
    if cfg.spheremesh==2562
      [pnt, tri] = icosahedron2562;
    elseif cfg.spheremesh==642
      [pnt, tri] = icosahedron642;
    elseif cfg.spheremesh==162
      [pnt, tri] = icosahedron162;
    elseif cfg.spheremesh==42
      [pnt, tri] = icosahedron42;
    elseif cfg.spheremesh==12
      [pnt, tri] = icosahedron;
    else
      error('invalid value for cfg.spheremesh');
    end
    % scale the sourcemodel sphere to the size of the headmodel sphere
    pnt = pnt * vol.r;
    pnt(:,1) = pnt(:,1) + vol.o(1);
    pnt(:,2) = pnt(:,2) + vol.o(2);
    pnt(:,3) = pnt(:,3) + vol.o(3);
  else
    fprintf('constructing brain surface from multiple sphere model\n');
    % first compute the indices of the relevant gradiometers.
    v    = char(vol.label);
    ind  = find(v(:,1)=='M');
    % create head-surface points from multisphere head-model.
    dir = grad.pnt(1:151,:)-vol.o(ind,:);
    len = sqrt(sum(dir.*dir,2));
    pnt = repmat((vol.r(ind)./len),1,3).*dir + vol.o(ind,:);
    % project headpoints and make triangulation
    prj  = elproj(pnt);
    tri  = delaunay(prj(:,1),prj(:,2));
  end
else
  fprintf('constructing brain surface from headshape file\n');
  shape = read_ctf_shape(cfg.headshape);
  pnt = shape.pnt;
  prj = elproj(pnt);
  tri = delaunay(prj(:,1), prj(:,2));
  [tri, pnt] = reducepatch(tri, pnt, 1000);
end

% correct the orientation of the triangles
sel=find(solid_angle(pnt,tri)<0);
tri(sel,:) = fliplr(tri(sel,:));

% shift the head surface inward with a certain amount
ori = normals(pnt, tri, 'vertex');
pnt = pnt + cfg.inwardshift * ori;
