function [grid, cfg] = prepare_dipole_grid(cfg, vol, sens);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that helps to prepare a regular grid
% this is the used in sourceanalysis and dipolefitting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: prepare_dipole_grid.m,v $
% Revision 1.9  2004/10/27 16:08:15  roboos
% added grid.subspace as one of the recognized fields that is kept
%
% Revision 1.8  2004/10/14 07:11:56  jansch
% changed outputted grid-positions to head-coordinates. added hidden option
% for setting a threshold to do inside-outside detection on the segmented volume
%
% Revision 1.7  2004/09/22 10:42:20  roboos
% added 'auto' defaults for x/y/zgrid
% convert brain segmentation to double before performing computations
% if the precomputed grid contains an lbex field, keep it
%
% Revision 1.6  2004/09/17 11:21:45  jansch
% inserted the possibility to create a grid, based on the cortex-segmentation
%
% Revision 1.5  2004/08/23 10:21:37  roboos
% added precomputed filters from grid AND from grid.avg
%
% Revision 1.4  2004/08/19 12:27:32  roboos
% fixed bug in incorrect copying of xgrid/ygrid/zgrid
%
% Revision 1.3  2004/08/19 06:30:52  roboos
% added extra consistency check for cfg.xgrid etc.
%
% Revision 1.2  2004/08/16 09:32:46  roboos
% added xgrid, ygrid, zgrid and dim to the output grid for consistency
%
% Revision 1.1  2004/06/28 08:59:38  roboos
% moved files from fieldtrip to fieldtrip/private
%
% Revision 1.2  2004/05/07 08:34:30  roberto
% added check for sensors being gradiometers and multisphere volume
%
% Revision 1.1  2004/03/01 11:00:04  roberto
% first entry in CVS, added precomputed filter to grid
%

% set the defaults
if ~isfield(cfg, 'xgrid'),            cfg.xgrid = 'auto';         end
if ~isfield(cfg, 'ygrid'),            cfg.ygrid = 'auto';         end
if ~isfield(cfg, 'zgrid'),            cfg.zgrid = 'auto';         end
if ~isfield(cfg, 'threshold'),        cfg.threshold = 0.1;        end

if ~isfield(cfg, 'grid') & ~isfield(cfg, 'mri'),   
  % determine the grid dimensions from the sensor 3D pointcloud extent
  if ischar(cfg.xgrid) & strcmp(cfg.xgrid, 'auto')
    cfg.xgrid = floor(min(sens.pnt(:,1))):cfg.resolution:ceil(max(sens.pnt(:,1)));
  end
  if ischar(cfg.ygrid) & strcmp(cfg.ygrid, 'auto')
    cfg.ygrid = floor(min(sens.pnt(:,2))):cfg.resolution:ceil(max(sens.pnt(:,2)));
  end
  if ischar(cfg.zgrid) & strcmp(cfg.zgrid, 'auto')
    cfg.zgrid = floor(min(sens.pnt(:,3))):cfg.resolution:ceil(max(sens.pnt(:,3)));
  end
  grid.xgrid = cfg.xgrid;
  grid.ygrid = cfg.ygrid;
  grid.zgrid = cfg.zgrid;
  grid.dim   = [length(grid.xgrid) length(grid.ygrid) length(grid.zgrid)];
elseif isfield(cfg, 'mri'),
  if ~strcmp(cfg.mri(end-7:end-4),'seg1'),
     warning('MRI-data is not recognized as a cortex-segmentation\n');
     mri           = read_fcdc_mri(cfg.mri);  
     mri.anatomy   = double(mri.anatomy);
     head          = mri.anatomy./max(mri.anatomy(:)) > cfg.threshold; %TO DO: find a way to compute a reasonable threshold
  elseif strcmp(cfg.mri(end-7:end-4),'seg1'),
     fprintf('constructing grid on cortex-segmented data\n');
     config.smooth = 5;
     mri           = segmentvolume(config,cfg.mri); 
     mri.gray      = double(mri.gray);
     head          = mri.gray./max(mri.gray(:)) > cfg.threshold;
  end
  ind                 = find(head(:));                         
  [X,Y,Z]             = ndgrid(mri.xgrid,mri.ygrid,mri.zgrid);              %create the grid in MRI-coordinates
  posmri              = [X(:) Y(:) Z(:) ones(length(X(:)),1)];
  posmri              = posmri(ind,:)';                                     %take only the inside voxels
  poshead             = mri.transform * posmri;                             %transform to head coordinates
  poshead             = poshead(1:3,:)';                    
  resolution          = cfg.resolution*10;%cfg.resolution in cm, resolution should be in mm.    
  xgrid               = floor(min(poshead(:,1))):resolution:ceil(max(poshead(:,1))); %create the grid in head-coordinates
  ygrid               = floor(min(poshead(:,2))):resolution:ceil(max(poshead(:,2))); %with 'consistent' x,y,z definitions
  zgrid               = floor(min(poshead(:,3))):resolution:ceil(max(poshead(:,3)));
  [X,Y,Z]             = ndgrid(xgrid,ygrid,zgrid);
  pos                 = [X(:) Y(:) Z(:) ones(length(X(:)),1)]';
  pos2                 = mri.transform \ pos;                                %transform to MRI-coordinates
  pos2                 = round(pos2(1:3,:))';
  posmri              = posmri(1:3,:)';
  [sel,inside,ind2]   = intersect(pos2,posmri,'rows');                       %do inside/outside detection

  grid.pos            = pos(1:3,:)'/10;                                             %convert to centimeters
  grid.xgrid          = xgrid/10; 
  grid.ygrid          = ygrid/10;
  grid.zgrid          = zgrid/10;
  grid.dim            = [length(grid.xgrid) length(grid.ygrid) length(grid.zgrid)];
  grid.inside         = inside;
  grid.outside        = setdiff(1:size(grid.pos,1),grid.inside)'; 

else
  % reuse as much of the prespecified grid as possible (but only known objects)
  if isfield(cfg.grid, 'pos')
    grid.pos = cfg.grid.pos;
  end
  if isfield(cfg.grid, 'dim')
    grid.dim = cfg.grid.dim;
  end
  if isfield(cfg.grid, 'xgrid')
    grid.xgrid = cfg.grid.xgrid;
    cfg.xgrid = grid.xgrid;	% for later reference
  end
  if isfield(cfg.grid, 'ygrid')
    grid.ygrid = cfg.grid.ygrid;
    cfg.ygrid = grid.ygrid;	% for later reference
  end
  if isfield(cfg.grid, 'zgrid')
    grid.zgrid = cfg.grid.zgrid;
    cfg.zgrid = grid.zgrid;	% for later reference
  end
  if isfield(cfg.grid, 'inside')
    grid.inside = cfg.grid.inside;
  end
  if isfield(cfg.grid, 'outside')
    grid.outside = cfg.grid.outside;
  end
  if isfield(cfg.grid, 'lbex')
    grid.lbex = cfg.grid.lbex;
  end
  if isfield(cfg.grid, 'subspace')
    grid.subspace = cfg.grid.subspace;
  end
  if isfield(cfg.grid, 'leadfield')
    grid.leadfield = cfg.grid.leadfield;
  end
  if isfield(cfg.grid, 'filter')
    grid.filter = cfg.grid.filter;
  elseif isfield(cfg.grid, 'avg')
    % the precomputed filters could also be in the average sub-structure
    if isfield(cfg.grid.avg, 'filter')
      grid.filter = cfg.grid.avg.filter;
    end
  end
  % this will remove all remaining unknown objects from the configuration
  cfg.grid = grid;
end

if ~isfield(grid, 'pos')
  % create the regular grid that will be scanned with a dipole
  [X, Y, Z] = ndgrid(cfg.xgrid, cfg.ygrid, cfg.zgrid);
  grid.pos   = [X(:) Y(:) Z(:)];
  grid.xgrid = cfg.xgrid;
  grid.ygrid = cfg.ygrid;
  grid.zgrid = cfg.zgrid;
end  

% determine the dipole locations inside the brain volume
if ~isfield(grid, 'inside') & ~isfield(grid, 'outside')
  if exist('sens', 'var') & exist('vol', 'var') & isfield(vol, 'r') & length(vol.r)>1 & isfield(sens, 'ori')
    fprintf('creating headsurface based on multisphere model and gradiometers\n');
    % make a triangulation of the head surface, using the multisphere model
    [pnt,tri] = head_surf(vol,sens);
    % determine the dipole positions that are inside the head surface
    tmp = bounding_mesh(grid.pos, pnt, tri);
    grid.inside  = find(tmp==1);
    grid.outside = find(tmp==0);
  else
    % use the default method of determining dipoles inside/outside the brain volume
    fprintf('determining dipoles inside brain compartment\n');
    [grid.inside, grid.outside] = find_inside_vol(grid.pos, vol);
  end
elseif ~isfield(grid, 'inside')
  grid.inside = setdiff(1:size(grid.pos,1), grid.outside);
elseif ~isfield(grid, 'outside')
  grid.outside = setdiff(1:size(grid.pos,1), grid.inside);
end
fprintf('%d dipoles inside, %d dipoles outside brain\n', length(grid.inside), length(grid.outside));
