function [Cf, Cr, Pr, Ntrials, cfg] = prepare_freq_matrices(cfg, freq);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBFUNCTION that converts a freq structure into Cf, Cr and Pr
% this is used in sourecanalysis
%
% This function returns data matrices with a channel order that is consistent 
% with the original channel order in the data.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Copyright (C) 2004, Robet Oostenveld
%
% $Log: prepare_freq_matrices.m,v $
% Revision 1.5  2005/03/03 10:59:29  roboos
% Added cfg as output argument and added selected chanels to the output
% cfg, so that the calling function knows which were selected. Renamed
% tbin into timebin.
%
% Revision 1.4  2005/02/16 15:13:50  roboos
% renamed cfg.refchannel into cfg.refchan, replaced detection of coh_refchan submethod for dics (now looking for presence of cfg.refchan instead of explicitely testing the submethod)
%
% Revision 1.3  2004/08/26 12:43:22  roboos
% fixed bug in backward-compatibility code for sgn/sgncmb
%
% Revision 1.2  2004/08/25 17:00:08  roboos
% changed from sgn/sgncmb to label/labelcmb, backward compatible
%
% Revision 1.1  2004/06/28 08:59:38  roboos
% moved files from fieldtrip to fieldtrip/private
%
% Revision 1.1  2004/04/08 15:51:20  roberto
% initial submissiion into cvs
%

% set the defaults
if ~isfield(cfg, 'channel'), cfg.channel = 'all'; end

Cf = [];
Cr = [];
Pr = [];

% the old freqanalysis (up to revision 1.9) used sgn and sgncmb
% for backward compatibility rename these
if isfield(freq, 'sgn')
  freq.label = freq.sgn;
  freq = rmfield(freq, 'sgn');
end
if isfield(freq, 'sgncmb')
  freq.labelcmb = freq.sgncmb;
  freq = rmfield(freq, 'sgncmb');
end

% select the latency of interest for time-frequency data
if strcmp(freq.dimord, 'sgncmb_frq_tim')
  tbin = nearest(freq.toi, cfg.latency);
  fprintf('selecting timeslice %d\n', tbin);
  freq.toi    = freq.toi(tbin);
  % remove all other latencies from the data structure and reduce the number of dimensions
  freq.powspctrm = squeeze(freq.powspctrm(:,:,tbin));
  freq.crsspctrm = squeeze(freq.crsspctrm(:,:,tbin));
  freq.dimord    = 'sgncmb_frq';
elseif strcmp(freq.dimord, 'rpt_sgncmb_frq_tim')
  tbin = nearest(freq.toi, cfg.latency);
  fprintf('selecting timeslice %d\n', tbin);
  freq.toi    = freq.toi(tbin);
  % remove all other latencies from the data structure and reduce the number of dimensions
  freq.powspctrm = squeeze(freq.powspctrm(:,:,:,tbin));
  freq.crsspctrm = squeeze(freq.crsspctrm(:,:,:,tbin));
  freq.dimord    = 'rpt_sgncmb_frq';
else
  tbin = [];
end

% the time-frequency latency has been squeezed away (see above) 
if strcmp(freq.dimord, 'sgncmb_frq')
  Ntrials = 1;
elseif strcmp(freq.dimord, 'rpt_sgncmb_frq')
  Ntrials = size(freq.powspctrm, 1);
else
  error('unrecognized dimord for frequency data');
end

% find the frequency of interest
fbin = nearest(freq.foi, cfg.frequency);

% find the index of each sensor channel into powspctrm
powspctrmindx = match_str(freq.label, cfg.channel);
Nchans        = length(powspctrmindx);

% find the index of each sensor channel combination into crsspctrm
crsspctrmindx = zeros(Nchans);
for sgncmblop=1:size(freq.labelcmb,1)
  ch1 = find(strcmp(cfg.channel, freq.labelcmb(sgncmblop,1)));
  ch2 = find(strcmp(cfg.channel, freq.labelcmb(sgncmblop,2)));
  if ~isempty(ch1) & ~isempty(ch2)
    % this square matrix contains the indices into the signal combinations
    crsspctrmindx(ch1,ch2) = sgncmblop;
  end
end

% this complex rearrangement of channel indices transforms the CSDs into a square matrix
if Ntrials==1
  Cf = complex(nan*zeros(Nchans,Nchans));
  % first use the complex conjugate for all reversed signal combinations 
  Cf(find(crsspctrmindx)) = freq.crsspctrm(crsspctrmindx(find(crsspctrmindx)), fbin);
  Cf = ctranspose(Cf);
  % and then get get the csd for all signal combinations
  Cf(find(crsspctrmindx)) = freq.crsspctrm(crsspctrmindx(find(crsspctrmindx)), fbin);
  % put the power on the diagonal
  Cf(find(eye(Nchans))) = freq.powspctrm(powspctrmindx, fbin);
else
  Cf  = complex(nan*zeros(Ntrials,Nchans,Nchans));
  tmp = complex(nan*zeros(Nchans,Nchans));
  for trial=1:Ntrials
    % first use the complex conjugate for all signal combinations reversed
    tmp(find(crsspctrmindx)) = freq.crsspctrm(trial, crsspctrmindx(find(crsspctrmindx)), fbin);
    tmp = ctranspose(tmp);
    % and then get get the csd for all signal combinations
    tmp(find(crsspctrmindx)) = freq.crsspctrm(trial, crsspctrmindx(find(crsspctrmindx)), fbin);
    % put the power on the diagonal
    tmp(find(eye(Nchans))) = freq.powspctrm(trial, powspctrmindx, fbin);
    Cf(trial,:,:) = tmp;
  end
end

% do a sanity check on the cross-spectral-density matrix
if any(isnan(Cf(:)))
  error('The cross-spectral-density matrix is not complete');
end

if isfield(cfg, 'refchan') & ~isempty(cfg.refchan) 
  % contruct the cross-spectral-density vector of the reference channel with all MEG channels
  tmpindx = match_str(freq.labelcmb(:,1), cfg.refchan);
  refindx = match_str(freq.labelcmb(tmpindx,2), cfg.channel);
  refindx = tmpindx(refindx);
  flipref = 0;
  if isempty(refindx)
    % first look in the second column, then in the first
    tmpindx = match_str(freq.labelcmb(:,2), cfg.refchan);
    refindx = match_str(freq.labelcmb(tmpindx,1), cfg.channel);
    refindx = tmpindx(refindx);
    flipref = 1;
  end
  if length(refindx)~=Nchans
    error('The cross-spectral-density with the reference channel is not complete');
  end
  if Ntrials==1
    Cr = freq.crsspctrm(refindx, fbin);
  else
    for trial=1:Ntrials
      Cr(trial,:) = freq.crsspctrm(trial, refindx, fbin);
    end
  end
  if flipref
    Cr = conj(Cr);
  end
  % obtain the power of the reference channel
  refindx = match_str(freq.label, cfg.refchan);
  if length(refindx)<1
    error('The reference channel was not found in powspctrm');
  elseif length(refindx)>1
    error('Multiple occurences of the reference channel found in powspctrm');
  end
  if Ntrials==1
    Pr = freq.powspctrm(refindx, fbin);
  else
    for trial=1:Ntrials
      Pr(trial) = freq.powspctrm(trial, refindx, fbin);
    end
    Pr = Pr(:);	% ensure that the first dimension contains the trials
  end
end

% update the configuration, so that the calling function exactly knows what was selected
if ~isempty(tbin),
  cfg.latency = freq.toi(tbin);
else
  cfg.latency = [];
end
cfg.frequency = freq.foi(fbin);
cfg.channel   = freq.label(powspctrmindx);

