function progress(varargin)

% PROGRESS shows a graphical or non-graphical progress indication similar
% to the standard Matlab WAITBAR function, but with the extra option of
% printing it in the command window as a plain text string or as a rotating
% dial. Alternatively, you can also specify it not to give feedback on the
% progress.
%
% Prior to the for-loop, you should call either
%   progress('init', 'none',   'Please wait...')
%   progress('init', 'gui',    'Please wait...')
%   progress('init', 'dial',   'Please wait...')      % rotating dial
%   progress('init', 'text',   'Please wait...')
%   progress('init', 'textcr', 'Please wait...')      % force cariage return
%   progress('init', 'textnl', 'Please wait...')      % force newline
%
% In each iteration of the for-loop, you should call either
%   progress(x)                                       % only show percentage
%   progress(x, 'Processing event %d from %d', i, N)  % show string, x=i/N
%
% After finishing the for-loop, you should call
%   progress('close')

% Copyright (C) 2004, Robert Oostenveld
%
% $Log: progress.m,v $
% Revision 1.2  2004/10/22 07:23:39  roboos
% improved help, comments and code layout
%
% Revision 1.1  2004/10/21 17:37:17  roboos
% new implementation, to be used to replace subfunction in  beamformer scan
% and in other fieldtrip functions
%

persistent p		% the previous value of the progress
persistent t		% type of feedback, string with none, gui, text, textcr, textnl
persistent h		% the gandle of the dialog (in case of type=gui)
persistent a		% the angle of the dial
persistent s		% the string containing the title

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin>1 & ischar(varargin{1}) & strcmp(varargin{1}, 'init')
  a = 0;
  p = 0;
  h = 0;
  % determine the type of feedback
  t = varargin{2};
  % determine the title of the dialog
  if nargin>2
    s = varargin{3};
  else
    s = '';
  end
  switch t
  case 'gui'
    % initialise the waitbar dialog
    if ~isempty(s)
      h = waitbar(0, s);
    else
      h = waitbar(0, 'Please wait');
    end
  case {'text', 'textnl', 'textcr'}
    if ~isempty(s)
      % print the title to the screen and go to the next line
      fprintf('%s\n', s)
    end
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif nargin==1 & ischar(varargin{1}) & strcmp(varargin{1}, 'close')
  switch t
  case 'gui'
    % close the waitbar dialog
    close(h);
  case {'textcr', 'dial'}
    % finish by going to the next line
    fprintf('\n');
  end
  % reset these to the defaults
  a = 0;
  h = 0;
  p = 0;
  t = 'none';
  s = '';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
  % only update the display if the current value is at least one percent larger than the previous value
  % but always update the display if a full fprintf format string is given
  if strcmp(t, 'gui') & (varargin{1}-p)<(0.01-100*eps)
    return;
   elseif  nargin==1 & (varargin{1}-p)<(0.01-100*eps) & ~strcmp(t, 'dial') 
    return;
  end

  % remember the value that will be displayed now for the next function call
  % and continue with updating the display
  p = varargin{1};

  switch t
  case 'gui'
    % update the the length of the bar in the waitbar dialog
    waitbar(varargin{1}, h);

  case 'dial'
    dial = '|/-\|/-\';
    if ~isempty(s)
      % print the title and draw a new hand of the rotating dial
      fprintf('\r%s %s', s, dial(1+a/45));
    else
      % draw a new hand of the rotating dial
      fprintf('\r%s', dial(1+a/45));
    end
    % increment the angle with 45 degrees
    a = a + 45;
    if a==360
      % reset the angle to 0 degrees
      a = 0;
    end

  case 'text'
    if nargin>1
      % print the string as it is
      fprintf(varargin{2:end});
    else
      fprintf('%6.2f %%\n', 100*varargin{1});
    end

  case 'textnl'
    if nargin>1
      % ensure that the string ends with a newline
      if length(varargin{2})>1 & all(varargin{2}((end-1):end) == '\r')
        varargin{2}((end-1):end) = '\n';
      elseif length(varargin{2})>1 & ~all(varargin{2}((end-1):end) == '\n')
        varargin{2}((end+1):(end+2)) = '\n';
      elseif length(varargin{2})<2
        varargin{2}((end+1):(end+2)) = '\n';
      end
      fprintf(varargin{2:end});
    else
      fprintf('%6.2f %%\n', 100*varargin{1});
    end

  case 'textcr'
    if nargin>1
      % ensure that the string ends with a cariage return
      if length(varargin{2})>1 & all(varargin{2}((end-1):end) == '\n')
        varargin{2}((end-1):end) = '\r';
      elseif length(varargin{2})>1 & ~all(varargin{2}((end-1):end) == '\r')
        varargin{2}((end+1):(end+2)) = '\r';
      elseif length(varargin{2})<2
        varargin{2}((end+1):(end+2)) = '\r';
      end
      fprintf(varargin{2:end});
    else
      fprintf('%6.2f %%\r', 100*varargin{1});
    end

  end % case gui, dial, text, textnl, textcr
end % updating the displayed value

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% some test code follows
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% progress('init', 'gui');  for i=1:100, progress(i/100, '%d/%d\r', i, 100); end; progress('close')
% progress('init', 'dial'); for i=1:100, progress(i/100, '%d/%d\r', i, 100); end; progress('close')
% progress('init', 'none'); for i=1:100, progress(i/100, '%d/%d\r', i, 100); end; progress('close')

% progress('init', 'text');   for i=1:100, progress(i/100, '%d/%d'  , i, 100); end; progress('close')
% progress('init', 'text');   for i=1:100, progress(i/100, '%d/%d\n', i, 100); end; progress('close')
% progress('init', 'text');   for i=1:100, progress(i/100, '%d/%d\r', i, 100); end; progress('close')

% progress('init', 'textnl'); for i=1:100, progress(i/100, '%d/%d'  , i, 100); end; progress('close')
% progress('init', 'textnl'); for i=1:100, progress(i/100, '%d/%d\n', i, 100); end; progress('close')
% progress('init', 'textnl'); for i=1:100, progress(i/100, '%d/%d\r', i, 100); end; progress('close')

% progress('init', 'textcr'); for i=1:100, progress(i/100, '%d/%d'  , i, 100); end; progress('close')
% progress('init', 'textcr'); for i=1:100, progress(i/100, '%d/%d\n', i, 100); end; progress('close')
% progress('init', 'textcr'); for i=1:100, progress(i/100, '%d/%d\r', i, 100); end; progress('close')


