function [interp] = sourceinterpolate(cfg, source, mri);

% SOURCEINTERPOLATE reslices and interpolates a source reconstruction
% or a statistical distribution as an overlay onto an anatomical MRI.
%
% The source volume should be in head-coordinates. The  anatomical MRI can
% be in any coordinates, but should contain a homogenous transformation
% matrix that defines how each MRI voxel can be expressed in head
% coordinates. The output volume will contain a resliced source and
% anatomical volume that can be plotted together, these are expressed in
% the original MRI coordinates.
%
% Use as
%   [interp] = sourceinterpolate(cfg, source, mri)   or
%   [interp] = sourceinterpolate(cfg, stat, mri)
% where
%   source is the output of SOURCEANALYSIS,
%   stat   is the output of SOURCESTATISTICS,
%   mri    is the output of READ_FCDC_MRI or an MRI filename
% and cfg is a structure with any of the following fields
%   cfg.interpmethod  = 'linear', 'cubic', 'nearest' or 'spline'
%   cfg.sourceunits   = 'mm' or 'cm' (default is 'cm')
%   cfg.mriunits      = 'mm' or 'cm' (default is 'mm')
%   cfg.downsample    = integer number, default 1 (no downsampling)
%
% See also SOURCEANALYSIS, SOURCESTATISTICS, READ_FCDC_MRI

% Undocumented options
%   cfg.voxecoord = 'yes' (default) or 'no' determines whether the
%   downsampled output anatomical MRI will have the x/y/zgrid converted or
%   the homogenous transformation matrix

% Copyright (C) 2003, Robert Oostenveld
%
% $Log: sourceinterpolate.m,v $
% Revision 1.20  2005/02/09 13:09:52  roboos
% explicitly emptied source.outside if keepinside=yes
%
% Revision 1.19  2005/02/08 11:45:00  roboos
% implemented cfg.mriunits (default mm)
% implemented cfg.keepinside (default yes)
% removed obsolete cfg.parameter
% changed handling of all volumes that should be interpolated (now through list with subfields)
% cleaned up code and help
%
% Revision 1.18  2004/10/13 14:11:30  roboos
% changed cfg.previous, now consistent over functions with try-statement
% and will also work if the input already had a cfg.previous
%
% Revision 1.17  2004/09/21 09:37:24  jansch
% fixed small bug due to yesterday's changes
%
% Revision 1.16  2004/09/20 09:32:15  roboos
% changed implementation of downsampling to use the external downsamplevolume function
%

% set the defaults
if ~isfield(cfg, 'interpmethod'); cfg.interpmethod = 'linear';  end
if ~isfield(cfg, 'sourceunits');  cfg.sourceunits  = 'cm';      end
if ~isfield(cfg, 'mriunits');     cfg.mriunits     = 'mm';      end
if ~isfield(cfg, 'downsample');   cfg.downsample   = 1;         end
if ~isfield(cfg, 'voxelcoord'),   cfg.voxelcoord   = 'yes';     end
if ~isfield(cfg, 'keepinside'),   cfg.keepinside   = 'yes';     end

if nargin<3
  error('3 input arguments required');
end

% assume that the source reconstruction is always in headcoordinates, that
% is what SOURCEANALYSIS and SOURCESTATS normally would produce
if isfield(source, 'transform') & ~all(all(source.transform==eye(4)))
  error('functional or statistical source volume is NOT in headcoordinates');
  % it would be possible to combine both transformation matrices into a
  % single one that links source and mri, but at the moment I have not
  % implemented that yet
end

% do the preprocessing of the source/stat structure
if ~isfield(source, 'pos') & isfield(source, 'avg') & isfield(source.avg, 'pos')
  % this is a source reconstruction with the old structure layout
  source.pos = source.avg.pos;
  source.avg = rmfield(source.avg, 'pos');
end
if ~isfield(source, 'inside') & isfield(source, 'avg') & isfield(source.avg, 'inside')
  % this is a source reconstruction with the old structure layout
  source.inside = source.avg.inside;
  source.avg = rmfield(source.avg, 'inside');
end
if ~isfield(source, 'outside') & isfield(source, 'avg') & isfield(source.avg, 'outside')
  % this is a source reconstruction with the old structure layout
  source.outside = source.avg.outside;
  source.avg = rmfield(source.avg, 'outside');
end

% convert the source grid locations into the same units as the mri grid locations
if strcmp(cfg.sourceunits, 'cm') & strcmp(cfg.mriunits, 'cm')
  scale = 1;
elseif strcmp(cfg.sourceunits, 'cm') & strcmp(cfg.mriunits, 'mm')
  scale = 10;
elseif strcmp(cfg.sourceunits, 'mm') & strcmp(cfg.mriunits, 'cm')
  scale = 0.1;
elseif strcmp(cfg.sourceunits, 'mm') & strcmp(cfg.mriunits, 'mm')
  scale = 1;
else
  error('unknown physical dimension of the source or mri (can be mm or cm)\n');
end
fprintf('converting source positions from %s into %s\n', cfg.sourceunits, cfg.mriunits);
source.pos   = 10*source.pos;
source.xgrid = 10*source.xgrid;
source.ygrid = 10*source.ygrid;
source.zgrid = 10*source.zgrid;

if ischar(mri)
  % read the anatomical MRI data from file
  filename = mri;
  fprintf('reading MRI from file\n');
  mri = read_fcdc_mri(filename);
end

% convert the anatomical MRI to double values which can be used for computation
mri8  = isa(mri.anatomy, 'uint8');
mri16 = isa(mri.anatomy, 'uint16');
mri.anatomy = double(mri.anatomy);

% downsample the anatomical MRI
interp = downsamplevolume(cfg, mri);

% convert the anatomical MRI back to original 8 or 16 bits unsigned integer values
if mri8
  interp.anatomy = uint8(interp.anatomy);
elseif mri16
  interp.anatomy = uint16(interp.anatomy);
end

% the source positions are by default given in HEAD coordinate system
fprintf('determining source grid in HEAD coordinate system\n');
pos = source.pos(source.inside,:);
xrange = [min(pos(:,1)) max(pos(:,1))];
yrange = [min(pos(:,2)) max(pos(:,2))];
zrange = [min(pos(:,3)) max(pos(:,3))];

if strcmp(cfg.keepinside, 'yes') & isfield(source, 'inside') & isfield(source, 'outside')
  % reformat the inside field to a full volume so that it can be interpolated just as the other volumes
  tmp = [];
  tmp(source.inside)  = 1;
  tmp(source.outside) = 0;
  source.inside  = tmp;
  source.outside = [];
else
  cfg.keepinside = 'no';
  % remove the inside and outside fields to prevent them from being interpolated and resliced
  if isfield(source, 'inside' ), source = rmfield(source, 'inside' ); end
  if isfield(source, 'outside'), source = rmfield(source, 'outside'); end
end

% determine interpolated/resliced grid positions in MRI coordinate system
fprintf('determining target grid in MRI coordinate system\n');
[interp.mri.X, interp.mri.Y, interp.mri.Z] = ndgrid(interp.xgrid, interp.ygrid, interp.zgrid);

% determine interpolated/resliced grid positions in HEAD coordinate system
fprintf('determining target grid in HEAD coordinate system\n');
interp.head.X = zeros(interp.dim);
interp.head.Y = zeros(interp.dim);
interp.head.Z = zeros(interp.dim);
% instead of warping all grid positions at the same time, do it per slice to save memory
nvox = interp.dim(1)*interp.dim(2);
for slice=1:interp.dim(3)
  fprintf('slice %d/%d\r', slice, interp.dim(3));
  sel = ((1:nvox) + (slice-1)*nvox)';
  pos = interp.transform * [interp.mri.X(sel) interp.mri.Y(sel) interp.mri.Z(sel) ones(length(sel), 1)]';
  pos = pos';
  interp.head.X(sel) = pos(:,1);
  interp.head.Y(sel) = pos(:,2);
  interp.head.Z(sel) = pos(:,3);
  clear pos
end
fprintf('\n');

% this field is not needed any more, remove it to save memory
interp = rmfield(interp, 'mri');

% select only grid locations for source interpolation that are approximately within the source volume
% this region of interest is used to restrict the interpolation to
sel =       interp.head.X>=xrange(1);
sel = sel & interp.head.X<=xrange(2);
sel = sel & interp.head.Y>=yrange(1);
sel = sel & interp.head.Y<=yrange(2);
sel = sel & interp.head.Z>=zrange(1);
sel = sel & interp.head.Z<=zrange(2);
fprintf('reduced interpolation volume to %3.1f%% (%d/%d)\n', 100*length(find(sel(:)))/length(sel(:)), length(find(sel(:))), length(sel(:)));

% define the list of known scalar volume parameters
param = {
  'inside'
  'anatomy'
  'mask'
  'zscore'
  'tscore'
  'prob'
  'avg.pow'
  'avg.coh'
  'avg.nai'
  'avg.noise'
  'var.pow'
  'var.coh'
  'var.nai'
  'var.noise'
  'sem.pow'
  'sem.coh'
  'sem.nai'
  'sem.noise'
  'avgA.pow'
  'avgA.coh'
  'avgA.nai'
  'avgA.noise'
  'avgB.pow'
  'avgB.coh'
  'avgB.nai'
  'avgB.noise'
  };

% reslice and interpolate all functional parameters onto the anatomical MRI
% grid
for i=1:length(param)
  if issubfield(source, param{i})
    fprintf('reslicing and interpolating %s\n', param{i});
    tmpold             = reshape(getsubfield(source, param{i}), source.dim);
    tmpnew             = zeros(interp.dim);
    tmpnew(find( sel)) = interpn(source.xgrid, source.ygrid, source.zgrid, tmpold, interp.head.X(find(sel)), interp.head.Y(find(sel)), interp.head.Z(find(sel)), cfg.interpmethod);
    tmpnew(find(~sel)) = nan;
    interp             = setsubfield(interp, param{i}, tmpnew);
  end
end

% this field is not needed any more, remove it to save memory
interp = rmfield(interp, 'head');

if strcmp(cfg.keepinside, 'yes')
  % restore the inside and outside fields in their proper formats
  tmp            = interp.inside(:);
  interp.inside  = find(abs(tmp-1)<=10*eps);
  interp.outside = setdiff(1:length(tmp), interp.inside)';
end

% add version information to the configuration
try
  % get the full name of the function
  cfg.version.name = mfilename('fullpath');
catch
  % required for compatibility with Matlab versions prior to release 13 (6.5)
  [st, i] = dbstack;
  cfg.version.name = st(i);
end
cfg.version.id = '$Id: sourceinterpolate.m,v 1.20 2005/02/09 13:09:52 roboos Exp $';

% remember the configuration of the input objects
cfg.previous = [];
try, cfg.previous{1} = source.cfg; end
try, cfg.previous{2} = mri.cfg; end

% remember the exact configuration details
interp.cfg = cfg;
