function topoplotER(cfg, data)                                               

% TOPOPLOTER plots topographic plots of event-related fields or potentials
% thar were computed using TIMELOCKANALYSIS.
%
% Use as
%   topoplotER(cfg, data)
% where data is produced by TIMELOCKANALYSIS, eventually after combining
% the planar gradients using COMBINEPLANAR. The cfg structure contains
% all configurations details for the plotting
%
% cfg.layout       = specification of the layout, see below
% cfg.baseline     = 'yes','no' or [time1 time2] (default = 'no')  
% cfg.baselinetype = 'absolute','relative' (default = 'absolute')  
% cfg.xlim         = [time1 time2] (default = full interval)
%                    Time interval for which to plot 
%                  = [time1 time2 time3 ... timeN] 
%                    Do N-1 plots: [time1 time2],[time2 time3] ... 
% cfg.zlim         = 'maxmin','absmax' or [ymin ymax] (default = 'absmax')
%                    Set the 'color limits'
% cfg.showlabels   = 'yes','no','markers','numbers' (default = 'no')
%                     Plot sensors labels/numbers or markers at each graph  
% cfg.colorbar     = 'yes','no' (default = 'yes')            
% cfg.showzlim     = 'yes','no' (default = 'yes')            
%                    Plot the color limits at the bottom 
% cfg.showxlim     = 'yes','no' (default = 'yes')
% cfg.fontsize     =  size of font (default = 8)             
% cfg.xparam       =  field defining the x-range (default = 'time')
% cfg.yparam       =  field y-defining the y-range (default = 'avg')
% 
% The layout defines what the position of each channel is. These
% positions are used for the interpolation of the field or potential.
% You can specify the layout in a variety of ways:
%  - you can give the name of an ascii layout file with extension *.lay
%  - you can give the name of an electrode file
%  - you can give an electrode definition, i.e. "elec" structure
%  - you can give a gradiometer definition, i.e. "grad" structure
% If you do not specify any of these and the data structure contains an
% electrode or gradiometer structure (common for MEG data, since the header
% of the MEG datafile contains the gradiometer information), that will be
% used for creating a layout. If you want to have more fine-grained control
% over the layout, you should create your own layout file.
% 
% See also TOPOPLOTFTR, MULTIPLOTER, MULTIPLOTTFR

% Copyrights (C) 2003, Ole Jensen
%
% $Log: topoplotER.m,v $
% Revision 1.6  2005/02/08 08:32:52  roboos
% fixed bug that was introduced in the last change (varargin->data)
%
% Revision 1.5  2005/02/07 17:33:55  roboos
% implemented support for new createlayout function for automatic construction of layout absed on gradiometer definitions that are present in the data, updated help and improved layout of the help
%
% Revision 1.4  2004/09/01 17:59:28  roboos
% added copyright statements to all filed
% added cfg.version to all functions that give configuration in their output
% added cfg.previous to all functions with input data containing configuration details
%

% set the defaults
if ~isfield(cfg,'baseline'),        cfg.baseline   = 'no'; end;
if ~isfield(cfg,'baselinetype'),    cfg.baselinetype   = 'absolute'; end;
if ~isfield(cfg,'zlim'),            cfg.zlim       = 'absmax'; end;
if ~isfield(cfg,'comment'),         cfg.comment    = strcat([date '\n' ]); end;
if ~isfield(cfg,'showlabels'),      cfg.showlabels   = 'no'; end;
if ~isfield(cfg,'showxlim'),        cfg.showxlim     = 'yes'; end;
if ~isfield(cfg,'showzlim'),        cfg.showzlim    = 'yes'; end;
if ~isfield(cfg,'colorbar'),        cfg.colorbar = 'yes'; end;
if ~isfield(cfg,'fontsize'),        cfg.fontsize    = 8; end;
if ~isfield(cfg,'yparam'),          cfg.yparam     = 'avg'; end;
if ~isfield(cfg,'xparam'),          cfg.xparam     = 'time'; end;

GRAPHCOLOR =['brgbkywrgbkywrgbkywrgbkyw'];

% get the data that should be plotted
P = eval(['data.',cfg.yparam]);
xvalues = eval(['data.',cfg.xparam]);

if ~isfield(cfg,'xlim'),        cfg.xlim  = [min(xvalues) max(xvalues)]; end;

% it always needs a specification like [begin end], even if you want to plot a single bin
if all(size(cfg.xlim)==[1 1])
  cfg.xlim = [cfg.xlim cfg.xlim];
end

% read or create the layout that will be used for plotting
if isfield(cfg, 'layout')
  lay = createlayout(cfg.layout);               % use specified layout
elseif isfield(cfg, 'layoutname')
  lay = createlayout(cfg.layoutname);           % idem, for backward compatibility
elseif isfield(data, 'grad')
  lay = createlayout(data.grad);                % create layout from gradiometer definition
elseif isfield(data, 'elec')
  lay = createlayout(data.elec);                % create layout from electrode definition
else
  lay = createlayout('CTF151s.lay');            % revert to the FCDC default
end

% convert the layout to Ole's style of variable names
X      = lay.prj(:,1);
Y      = lay.prj(:,2);
Width  = lay.box(:,1);
Height = lay.box(:,2);
Lbl    = lay.label;

if strcmp(cfg.baseline,'yes')
  % default is to use pretrigger interval for baseline
  cfg.baseline = [];
  cfg.baseline(1) = min(data.toi);
  cfg.baseline(2) = 0;
end

% compute and apply the baseline correction
if ~isempty(cfg.baseline) & ~strcmp(cfg.baseline, 'no')
  tidx = find(xvalues >= cfg.baseline(1) & xvalues <= cfg.baseline(2)); 
  if ~isempty(tidx)
    tmp = mean(P(:,tidx),2);
    for l=1:size(P,1)
      if strcmp(cfg.baselinetype,'absolute')
        P(l,:) = P(l,:) - tmp(l);
      elseif strcmp(cfg.baselinetype,'relative')
        P(l,:) = 100*(P(l,:) - tmp(l))./tmp(l);
      end
    end
  end
end

% find matching channels in the data and the layout
[sensorOrder, dataOrder] = match_str(Lbl, data.label);
fprintf('selected %d channels for plotting\n', length(sensorOrder));

% this loop creates multiple subplots in a row, each with a different
% timeselection
for k=1:length(cfg.xlim)-1 
  if length(cfg.xlim)-1 > 1 
    subplot(ceil((length(cfg.xlim)-1)/6),6,k)
  end
  tidx = find(cfg.xlim(k) <= xvalues & (cfg.xlim(k+1) > xvalues));
  if length(tidx)>1
      V = mean(P(:,tidx),2);
  elseif length(tidx)==1
      V = P(:,tidx);
  elseif length(tidx)==0
      V = P;
  end;    
  topoplot(cfg,X(sensorOrder),Y(sensorOrder),V(dataOrder),Lbl(sensorOrder))
  if strcmp(cfg.showxlim,'yes')
    title(sprintf('Time:\n%.2e to %.2e ',cfg.xlim(k),cfg.xlim(k+1)),'fontsize',cfg.fontsize)
  end
  drawnow
end
