function [result, M] = warp_el(input, input_lab, target, target_lab, method)

% WARP_EL determine intermediate electrode positions using warping (deformation)
% After matching the corresponding electrode labels, the input electrodes are 
% warped to match the target electrodes
%
% Use as
%   [result, M] = warp_el(input, input_lab, target, target_lab, method)
% where
%    input          contains the Nx3 measured electrode positions
%    input_lab      contains the N labels of the measured electrode positions
%    target         contains the Nx3 template electrode positions
%    target_lab     contains the N labels of the template electrode positions
%    method         should be empty or any of 'nonlin1', 'nonlin2' ... 'nonlin5'
%
% The default is a linear warp with rescaling in each dimension, optional you
% can select a nonlinear warp of the 1st (affine) up to the 5th order. Besides the
% warped result, this function also returns the transformation vector/matrix M.
%
% This function depends on the OPTIM and WARPING toolboxes.
%
% See also WARP_PNT, DIST_PNT, WARP3D, WARPING

% Copyright (C) 2000-2004, Robert Oostenveld
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: warp_el.m,v $
% Revision 1.4  2004/05/19 09:57:07  roberto
% added GPL copyright statement, added CVS log item
%

global fb;

if nargin<5
   method='';
end

% find the matching electrode labels
[sel1, sel2] = match_str(input_lab, target_lab);

% select the electrodes wich match in both electrode clouds and start warping them
% the electrodes in pos1 will be warped, minimizing the distance between pos1 and pos2
pos1 = input(sel1,:);
pos2 = target(sel2,:);

% if fb
%   disp('using electrode selection:');
%   disp([sel1 sel2]);
% end

% set the options for the minimalisation routine
options = optimset('fminunc');
options = optimset(options, 'Display', 'off');
options = optimset(options, 'MaxIter', 1500);
% options = optimset(options, 'MaxFunEvals', '1000*numberOfVariables');
options = optimset(options, 'TolFun', 1e-4);
options = optimset(options, 'LargeScale', 'off');

if fb; fprintf('distance = %f\n', warpfun([0 0 0 0 0 0], pos1, pos2, 'rigidbody')); end

% do a rigid-body transformation (6 parameters)
if fb; disp('rigidbody...'); end
ri = [0 0 0 0 0 0];
rf = fminunc('warpfun', ri, options, pos1, pos2, 'rigidbody');
if fb; fprintf('distance = %f\n', warpfun(rf, pos1, pos2, 'rigidbody')); end

% do a rigid-body + global rescaling transformation (7 parameters)
if fb; disp('rigidbody + global rescaling...'); end
gi = [rf 1];
gf = fminunc('warpfun', gi, options, pos1, pos2, 'globalrescale');
if fb; fprintf('distance = %f\n', warpfun(gf, pos1, pos2, 'globalrescale')); end

% do a rigid-body + individual rescaling transformation (9 parameters)
if fb; disp('rigidbody + individual rescaling...'); end
ti = [gf gf(7) gf(7)];
tf = fminunc('warpfun', ti, options, pos1, pos2, 'traditional');
if fb; fprintf('distance = %f\n', warpfun(tf, pos1, pos2, 'traditional')); end

% apply the warp to all electrodes of the input set
result = warp3d(tf,  input, 'traditional');
M = tf;

% start with the optional non-linear warping
% using the traditional warping result as starting point

if strcmp(method, 'nonlin1') | strcmp(method, 'nonlin2') | strcmp(method, 'nonlin3') | strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
% do a first order nonlinear transformation, 
if fb; disp('1st order nonlinear...'); end
e1i = traditional(tf);
e1i = [e1i(1:3,4) e1i(1:3,1:3)];	% reshuffle from homogenous to warp3d
e1f = fminunc('warpfun', e1i, options, pos1, pos2);
result = warp3d(e1f, input);
M = e1f;
if fb; fprintf('distance = %f\n', warpfun(e1f, pos1, pos2, 'nonlinear')); end

if strcmp(method, 'nonlin2') | strcmp(method, 'nonlin3') | strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
% do a second order nonlinear transformation, 
if fb; disp('2nd order nonlinear...'); end
e2i = [e1f zeros(3,6)];
e2f = fminunc('warpfun', e2i, options, pos1, pos2);
result = warp3d(e2f, input);
M = e2f;
if fb; fprintf('distance = %f\n', warpfun(e2f, pos1, pos2, 'nonlinear')); end

if strcmp(method, 'nonlin3') | strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
% do a third order nonlinear transformation, 
if fb; disp('3rd order nonlinear...'); end
e3i = [e2f zeros(3,10)];
e3f = fminunc('warpfun', e3i, options, pos1, pos2);
result = warp3d(e3f, input);
M = e3f;
if fb; fprintf('distance = %f\n', warpfun(e3f, pos1, pos2, 'nonlinear')); end

if strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
% do a third order nonlinear transformation, 
if fb; disp('4th order nonlinear...'); end
e4i = [e3f zeros(3,15)];
e4f = fminunc('warpfun', e4i, options, pos1, pos2);
result = warp3d(e4f, input);
M = e4f;
if fb; fprintf('distance = %f\n', warpfun(e4f, pos1, pos2, 'nonlinear')); end

if strcmp(method, 'nonlin5')
% do a third order nonlinear transformation, 
if fb; disp('5th order nonlinear...'); end
e5i = [e4f zeros(3,21)];
e5f = fminunc('warpfun', e5i, options, pos1, pos2);
result = warp3d(e5f, input);
M = e5f;
if fb; fprintf('distance = %f\n', warpfun(e5f, pos1, pos2, 'nonlinear')); end

end	% nonlin5
end	% nonlin4
end	% nonlin3
end	% nonlin2
end	% nonlin1
