function [result, M] = warp_pnt(input, target, method)

% WARP_PNT determine intermediate positions using warping (deformation)
% the input cloud of points is warped to match the target.
% The strategy is to start with simpelest linear warp, followed by a more
% elaborate linear warp, which then is followed by the nonlinear warps up
% to the desired order.
%
% [result, M] = warp_pnt(input, target, method)
%     input          contains the Nx3 measured 3D positions
%     target         contains the Nx3 template 3D positions
%     method         should be empty or any of 'nonlin1', 'nonlin2' ... 'nonlin5'
%
% The default is a linear warp with rescaling in each dimension, optional you
% can select a nonlinear warp of the 1st (affine) up to the 5th order.
%
% This function depends on the OPTIM and WARPING toolboxes.
%
% See also DIST_PNT, WARP_EL, WARPING

% Copyright (C) 2000-2005, Robert Oostenveld
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: warp_pnt.m,v $
% Revision 1.4  2005/03/21 15:43:42  roboos
% fixed bug in output for nonlinear warping
% added support for rigidbody or globalrescale warp
%
% Revision 1.3  2004/05/19 09:57:08  roberto
% added GPL copyright statement, added CVS log item
%

global fb;

if nargin<3
  method='traditional';
end

pos1 = input;
pos2 = target;

% set the options for the minimalisation routine
options = optimset('fminunc');
options = optimset(options, 'Display', 'off');
options = optimset(options, 'MaxIter', 1500);
% options = optimset(options, 'MaxFunEvals', '1000*numberOfVariables');
options = optimset(options, 'TolFun', 1e-4);
options = optimset(options, 'LargeScale', 'off');

if fb; fprintf('distance = %f\n', warpfun([0 0 0 0 0 0], pos1, pos2, 'rigidbody')); end

% do a rigid-body transformation (6 parameters)
if fb; disp('rigidbody...'); end
ri = [0 0 0 0 0 0];
rf = fminunc('warpfun', ri, options, pos1, pos2, 'rigidbody');
if fb; fprintf('distance = %f\n', warpfun(rf, pos1, pos2, 'rigidbody')); end
if strcmp(method, 'rigidbody')
  M = rf;
  result = warp3d(M, input, method);
  return;
end

% do a rigid-body + global rescaling transformation (7 parameters)
if fb; disp('rigidbody + global rescaling...'); end
gi = [rf 1];
gf = fminunc('warpfun', gi, options, pos1, pos2, 'globalrescale');
if fb; fprintf('distance = %f\n', warpfun(gf, pos1, pos2, 'globalrescale')); end
if strcmp(method, 'globalrescale')
  M = gf;
  result = warp3d(M, input, method);
  return;
end

% do a rigid-body + individual rescaling transformation (9 parameters)
if fb; disp('rigidbody + individual rescaling...'); end
ti = [gf gf(7) gf(7)];
tf = fminunc('warpfun', ti, options, pos1, pos2, 'traditional');
if fb; fprintf('distance = %f\n', warpfun(tf, pos1, pos2, 'traditional')); end
if strcmp(method, 'traditional')
  M = tf;
  result = warp3d(M, input, method);
  return;
end

% start with the optional non-linear warping
% using the traditional warping result as starting point

if strcmp(method, 'nonlin1') | strcmp(method, 'nonlin2') | strcmp(method, 'nonlin3') | strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
  % do a first order nonlinear transformation,
  if fb; disp('1st order nonlinear...'); end
  e1i = traditional(tf);
  e1i = [e1i(1:3,4) e1i(1:3,1:3)];	% reshuffle from homogenous to warp3d
  e1f = fminunc('warpfun', e1i, options, pos1, pos2);
  if fb; fprintf('distance = %f\n', warpfun(e1f, pos1, pos2, 'nonlinear')); end

  if strcmp(method, 'nonlin2') | strcmp(method, 'nonlin3') | strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
    % do a second order nonlinear transformation,
    if fb; disp('2nd order nonlinear...'); end
    e2i = [e1f zeros(3,6)];
    e2f = fminunc('warpfun', e2i, options, pos1, pos2);
    if fb; fprintf('distance = %f\n', warpfun(e2f, pos1, pos2, 'nonlinear')); end

    if strcmp(method, 'nonlin3') | strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
      % do a third order nonlinear transformation,
      if fb; disp('3rd order nonlinear...'); end
      e3i = [e2f zeros(3,10)];
      e3f = fminunc('warpfun', e3i, options, pos1, pos2);
      if fb; fprintf('distance = %f\n', warpfun(e3f, pos1, pos2, 'nonlinear')); end

      if strcmp(method, 'nonlin4') | strcmp(method, 'nonlin5')
        % do a third order nonlinear transformation,
        if fb; disp('4th order nonlinear...'); end
        e4i = [e3f zeros(3,10)];
        e4f = fminunc('warpfun', e4i, options, pos1, pos2);
        if fb; fprintf('distance = %f\n', warpfun(e4f, pos1, pos2, 'nonlinear')); end

        if strcmp(method, 'nonlin5')
          % do a third order nonlinear transformation,
          if fb; disp('5th order nonlinear...'); end
          e5i = [e4f zeros(3,10)];
          e5f = fminunc('warpfun', e5i, options, pos1, pos2);
          if fb; fprintf('distance = %f\n', warpfun(e5f, pos1, pos2, 'nonlinear')); end

          M = e5f;
          result = warp3d(M, input);
          return
        end	% nonlin5
        M = e4f;
        result = warp3d(M, input);
        return
      end	% nonlin4
      M = e3f;
      result = warp3d(M, input);
      return
    end	% nonlin3
    M = e2f;
    result = warp3d(M, input);
    return
  end	% nonlin2
  M = e1f;
  result = warp3d(M, input);
  return
end	% nonlin1
